@@ -13,34 +13,35 @@
 #include "test/cctest/cctest.h"
 #include "test/cctest/compiler/codegen-tester.h"
 #include "test/cctest/compiler/graph-and-builders.h"
-#include "test/cctest/compiler/value-helper.h"
+#include "test/cctest/compiler/js-heap-broker-base.h"
+#include "test/common/value-helper.h"
 
 namespace v8 {
 namespace internal {
 namespace compiler {
 
 class RepresentationChangerTester : public HandleAndZoneScope,
-                                    public GraphAndBuilders {
+                                    public GraphAndBuilders,
+                                    public JSHeapBrokerTestBase {
  public:
   explicit RepresentationChangerTester(int num_parameters = 0)
       : HandleAndZoneScope(kCompressGraphZone),
         GraphAndBuilders(main_zone()),
+        JSHeapBrokerTestBase(main_isolate(), main_zone()),
         javascript_(main_zone()),
         jsgraph_(main_isolate(), main_graph_, &main_common_, &javascript_,
                  &main_simplified_, &main_machine_),
-        broker_(main_isolate(), main_zone()),
-        changer_(&jsgraph_, &broker_, nullptr) {
+        changer_(&jsgraph_, broker(), nullptr) {
     Node* s = graph()->NewNode(common()->Start(num_parameters));
     graph()->SetStart(s);
   }
 
   JSOperatorBuilder javascript_;
   JSGraph jsgraph_;
-  JSHeapBroker broker_;
   RepresentationChanger changer_;
 
   Isolate* isolate() { return main_isolate(); }
-  Graph* graph() { return main_graph_; }
+  TFGraph* graph() { return main_graph_; }
   CommonOperatorBuilder* common() { return &main_common_; }
   JSGraph* jsgraph() { return &jsgraph_; }
   RepresentationChanger* changer() { return &changer_; }
@@ -76,7 +77,7 @@ class RepresentationChangerTester : public HandleAndZoneScope,
     CHECK_FLOAT_EQ(expected, fval);
   }
 
-  void CheckHeapConstant(Node* n, HeapObject expected) {
+  void CheckHeapConstant(Node* n, Tagged<HeapObject> expected) {
     HeapObjectMatcher m(n);
     CHECK(m.HasResolvedValue());
     CHECK_EQ(expected, *m.ResolvedValue());
@@ -151,7 +152,7 @@ TEST(ToTagged_constant) {
   RepresentationChangerTester r;
 
   for (double i : ValueHelper::float64_vector()) {
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kFloat64, Type::None(), use,
@@ -160,7 +161,7 @@ TEST(ToTagged_constant) {
   }
 
   for (int i : ValueHelper::int32_vector()) {
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kWord32, Type::Signed32(), use,
@@ -169,7 +170,7 @@ TEST(ToTagged_constant) {
   }
 
   for (uint32_t i : ValueHelper::uint32_vector()) {
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kWord32, Type::Unsigned32(), use,
@@ -182,7 +183,7 @@ TEST(ToFloat64_constant) {
   RepresentationChangerTester r;
 
   for (double i : ValueHelper::float64_vector()) {
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kTagged, Type::None(), use,
@@ -191,7 +192,7 @@ TEST(ToFloat64_constant) {
   }
 
   for (int i : ValueHelper::int32_vector()) {
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kWord32, Type::Signed32(), use,
@@ -200,7 +201,7 @@ TEST(ToFloat64_constant) {
   }
 
   for (uint32_t i : ValueHelper::uint32_vector()) {
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kWord32, Type::Unsigned32(), use,
@@ -209,7 +210,7 @@ TEST(ToFloat64_constant) {
   }
 
   {
-    Node* n = r.jsgraph()->Constant(0);
+    Node* n = r.jsgraph()->ConstantNoHole(0);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kWord64, Type::Range(0, 0, r.zone()), use,
@@ -231,7 +232,7 @@ TEST(ToFloat32_constant) {
   RepresentationChangerTester r;
 
   for (double i : ValueHelper::float32_vector()) {
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kTagged, Type::None(), use,
@@ -241,7 +242,7 @@ TEST(ToFloat32_constant) {
 
   for (int i : ValueHelper::int32_vector()) {
     if (!IsFloat32Int32(i)) continue;
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kWord32, Type::Signed32(), use,
@@ -251,7 +252,7 @@ TEST(ToFloat32_constant) {
 
   for (uint32_t i : ValueHelper::uint32_vector()) {
     if (!IsFloat32Uint32(i)) continue;
-    Node* n = r.jsgraph()->Constant(i);
+    Node* n = r.jsgraph()->ConstantNoHole(i);
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
         n, MachineRepresentation::kWord32, Type::Unsigned32(), use,
@@ -265,7 +266,7 @@ TEST(ToInt32_constant) {
   {
     FOR_INT32_INPUTS(i) {
       const double value = static_cast<double>(i);
-      Node* n = r.jsgraph()->Constant(value);
+      Node* n = r.jsgraph()->ConstantNoHole(value);
       NodeProperties::SetType(n, Type::Constant(value, r.zone()));
       Node* use = r.Return(n);
       Node* c = r.changer()->GetRepresentationFor(
@@ -280,7 +281,7 @@ TEST(ToUint32_constant) {
   RepresentationChangerTester r;
   FOR_UINT32_INPUTS(i) {
     const double value = static_cast<double>(i);
-    Node* n = r.jsgraph()->Constant(value);
+    Node* n = r.jsgraph()->ConstantNoHole(value);
     NodeProperties::SetType(n, Type::Constant(value, r.zone()));
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
@@ -294,7 +295,7 @@ TEST(ToInt64_constant) {
   RepresentationChangerTester r;
   FOR_INT32_INPUTS(i) {
     const double value = static_cast<double>(i);
-    Node* n = r.jsgraph()->Constant(value);
+    Node* n = r.jsgraph()->ConstantNoHole(value);
     NodeProperties::SetType(n, Type::Constant(value, r.zone()));
     Node* use = r.Return(n);
     Node* c = r.changer()->GetRepresentationFor(
