@@ -3,7 +3,9 @@
 // found in the LICENSE file.
 
 #include "src/api/api-inl.h"
+#include "src/common/globals.h"
 #include "src/execution/isolate.h"
+#include "src/flags/flags.h"
 #include "src/heap/array-buffer-sweeper.h"
 #include "src/heap/heap-inl.h"
 #include "src/heap/spaces.h"
@@ -35,8 +37,8 @@ bool IsTracked(i::Heap* heap, i::ArrayBufferExtension* extension) {
   return in_young || in_old;
 }
 
-bool IsTracked(i::Heap* heap, i::JSArrayBuffer buffer) {
-  return IsTracked(heap, buffer.extension());
+bool IsTracked(i::Heap* heap, i::Tagged<i::JSArrayBuffer> buffer) {
+  return IsTracked(heap, buffer->extension());
 }
 
 }  // namespace
@@ -49,117 +51,135 @@ namespace heap {
 // moving the objects through various spaces during GC phases.
 
 TEST(ArrayBuffer_OnlyMC) {
-  FLAG_concurrent_array_buffer_sweeping = false;
+  v8_flags.concurrent_array_buffer_sweeping = false;
 
   ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
   Heap* heap = reinterpret_cast<Isolate*>(isolate)->heap();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(
+      CcTest::heap());
 
   ArrayBufferExtension* extension;
   {
     v8::HandleScope handle_scope(isolate);
     Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 100);
-    Handle<JSArrayBuffer> buf = v8::Utils::OpenHandle(*ab);
+    DirectHandle<JSArrayBuffer> buf = v8::Utils::OpenDirectHandle(*ab);
     extension = buf->extension();
-    CHECK(FLAG_single_generation ? IsTrackedOld(heap, extension)
-                                 : IsTrackedYoung(heap, extension));
-    heap::GcAndSweep(heap, OLD_SPACE);
+    CHECK(v8_flags.single_generation ? IsTrackedOld(heap, extension)
+                                     : IsTrackedYoung(heap, extension));
+    heap::InvokeAtomicMajorGC(heap);
     CHECK(IsTrackedOld(heap, extension));
-    heap::GcAndSweep(heap, OLD_SPACE);
+    heap::InvokeAtomicMajorGC(heap);
     CHECK(IsTrackedOld(heap, extension));
   }
-  heap::GcAndSweep(heap, OLD_SPACE);
+  heap::InvokeAtomicMajorGC(heap);
   CHECK(!IsTracked(heap, extension));
 }
 
 TEST(ArrayBuffer_OnlyScavenge) {
-  if (FLAG_single_generation) return;
-  FLAG_concurrent_array_buffer_sweeping = false;
+  if (v8_flags.single_generation) return;
+  v8_flags.concurrent_array_buffer_sweeping = false;
 
   ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
   Heap* heap = reinterpret_cast<Isolate*>(isolate)->heap();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(
+      CcTest::heap());
 
   ArrayBufferExtension* extension;
   {
     v8::HandleScope handle_scope(isolate);
     Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 100);
-    Handle<JSArrayBuffer> buf = v8::Utils::OpenHandle(*ab);
+    DirectHandle<JSArrayBuffer> buf = v8::Utils::OpenDirectHandle(*ab);
     extension = buf->extension();
     CHECK(IsTrackedYoung(heap, extension));
-    heap::GcAndSweep(heap, NEW_SPACE);
+    heap::InvokeAtomicMinorGC(heap);
     CHECK(IsTrackedYoung(heap, extension));
-    heap::GcAndSweep(heap, NEW_SPACE);
-    CHECK(IsTrackedOld(heap, extension));
-    heap::GcAndSweep(heap, NEW_SPACE);
+    heap::InvokeAtomicMajorGC(heap);
     CHECK(IsTrackedOld(heap, extension));
   }
-  heap::GcAndSweep(heap, OLD_SPACE);
+  heap::InvokeAtomicMajorGC(heap);
   CHECK(!IsTracked(heap, extension));
 }
 
 TEST(ArrayBuffer_ScavengeAndMC) {
-  if (FLAG_single_generation) return;
-  FLAG_concurrent_array_buffer_sweeping = false;
+  if (v8_flags.single_generation) return;
+  v8_flags.concurrent_array_buffer_sweeping = false;
 
   ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
   Heap* heap = reinterpret_cast<Isolate*>(isolate)->heap();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(
+      CcTest::heap());
 
   ArrayBufferExtension* extension;
   {
     v8::HandleScope handle_scope(isolate);
     Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 100);
-    Handle<JSArrayBuffer> buf = v8::Utils::OpenHandle(*ab);
+    DirectHandle<JSArrayBuffer> buf = v8::Utils::OpenDirectHandle(*ab);
     extension = buf->extension();
     CHECK(IsTrackedYoung(heap, extension));
-    heap::GcAndSweep(heap, NEW_SPACE);
+    heap::InvokeAtomicMinorGC(heap);
     CHECK(IsTrackedYoung(heap, extension));
-    heap::GcAndSweep(heap, NEW_SPACE);
+    heap::InvokeAtomicMajorGC(heap);
     CHECK(IsTrackedOld(heap, extension));
-    heap::GcAndSweep(heap, OLD_SPACE);
-    CHECK(IsTrackedOld(heap, extension));
-    heap::GcAndSweep(heap, NEW_SPACE);
+    heap::InvokeAtomicMinorGC(heap);
     CHECK(IsTrackedOld(heap, extension));
   }
-  heap::GcAndSweep(heap, OLD_SPACE);
+  heap::InvokeAtomicMinorGC(heap);
+  CHECK(IsTrackedOld(heap, extension));
+  heap::InvokeAtomicMajorGC(heap);
   CHECK(!IsTracked(heap, extension));
 }
 
 TEST(ArrayBuffer_Compaction) {
-  if (!FLAG_compact) return;
+  if (!v8_flags.compact) return;
   ManualGCScope manual_gc_scope;
-  FLAG_manual_evacuation_candidates_selection = true;
-  FLAG_concurrent_array_buffer_sweeping = false;
+  heap::ManualEvacuationCandidatesSelectionScope
+      manual_evacuation_candidate_selection_scope(manual_gc_scope);
+  v8_flags.concurrent_array_buffer_sweeping = false;
   CcTest::InitializeVM();
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
   Heap* heap = reinterpret_cast<Isolate*>(isolate)->heap();
   heap::AbandonCurrentlyFreeMemory(heap->old_space());
 
-  v8::HandleScope handle_scope(isolate);
-  Local<v8::ArrayBuffer> ab1 = v8::ArrayBuffer::New(isolate, 100);
-  Handle<JSArrayBuffer> buf1 = v8::Utils::OpenHandle(*ab1);
-  CHECK(IsTracked(heap, *buf1));
-  heap::GcAndSweep(heap, NEW_SPACE);
-  heap::GcAndSweep(heap, NEW_SPACE);
+  Global<v8::ArrayBuffer> ab1_global;
+  PageMetadata* page_before_gc;
+  {
+    v8::HandleScope handle_scope(isolate);
+    Local<v8::ArrayBuffer> ab1 = v8::ArrayBuffer::New(isolate, 100);
+    IndirectHandle<JSArrayBuffer> buf1 = v8::Utils::OpenIndirectHandle(*ab1);
+    CHECK(IsTracked(heap, *buf1));
+    heap::InvokeAtomicMajorGC(heap);
 
-  Page* page_before_gc = Page::FromHeapObject(*buf1);
-  heap::ForceEvacuationCandidate(page_before_gc);
-  CHECK(IsTracked(heap, *buf1));
+    page_before_gc = PageMetadata::FromHeapObject(*buf1);
+    heap::ForceEvacuationCandidate(page_before_gc);
+    CHECK(IsTracked(heap, *buf1));
+    ab1_global.Reset(isolate, ab1);
+  }
+  {
+    // We need to invoke GC without stack, otherwise no compaction is
+    // performed.
+    DisableConservativeStackScanningScopeForTesting no_stack_scanning(heap);
+    heap::InvokeMajorGC(heap);
+  }
 
-  CcTest::CollectAllGarbage();
+  {
+    v8::HandleScope scope(isolate);
+    IndirectHandle<JSArrayBuffer> buf1 =
+        v8::Utils::OpenHandle(*ab1_global.Get(isolate));
+    PageMetadata* page_after_gc = PageMetadata::FromHeapObject(*buf1);
+    CHECK(IsTracked(heap, *buf1));
 
-  Page* page_after_gc = Page::FromHeapObject(*buf1);
-  CHECK(IsTracked(heap, *buf1));
-
-  CHECK_NE(page_before_gc, page_after_gc);
+    CHECK_NE(page_before_gc, page_after_gc);
+  }
 }
 
 TEST(ArrayBuffer_UnregisterDuringSweep) {
@@ -173,10 +193,10 @@ TEST(ArrayBuffer_UnregisterDuringSweep) {
 // Disable verify-heap since it forces sweeping to be completed in the
 // epilogue of the GC.
 #ifdef VERIFY_HEAP
-  i::FLAG_verify_heap = false;
+  i::v8_flags.verify_heap = false;
 #endif  // VERIFY_HEAP
   ManualGCScope manual_gc_scope;
-  i::FLAG_concurrent_array_buffer_sweeping = false;
+  i::v8_flags.concurrent_array_buffer_sweeping = false;
 
   CcTest::InitializeVM();
   LocalContext env;
@@ -185,33 +205,33 @@ TEST(ArrayBuffer_UnregisterDuringSweep) {
   {
     v8::HandleScope handle_scope(isolate);
     Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 100);
-    Handle<JSArrayBuffer> buf = v8::Utils::OpenHandle(*ab);
+    DirectHandle<JSArrayBuffer> buf = v8::Utils::OpenDirectHandle(*ab);
 
     {
       v8::HandleScope new_handle_scope(isolate);
       // Allocate another buffer on the same page to force processing a
       // non-empty set of buffers in the last GC.
       Local<v8::ArrayBuffer> ab2 = v8::ArrayBuffer::New(isolate, 100);
-      Handle<JSArrayBuffer> buf2 = v8::Utils::OpenHandle(*ab2);
+      DirectHandle<JSArrayBuffer> buf2 = v8::Utils::OpenDirectHandle(*ab2);
       CHECK(IsTracked(heap, *buf));
-      heap::GcAndSweep(heap, NEW_SPACE);
+      heap::InvokeAtomicMinorGC(heap);
       CHECK(IsTracked(heap, *buf));
-      heap::GcAndSweep(heap, NEW_SPACE);
+      heap::InvokeAtomicMinorGC(heap);
       CHECK(IsTracked(heap, *buf));
       CHECK(IsTracked(heap, *buf2));
     }
 
-    CcTest::CollectGarbage(OLD_SPACE);
+    heap::InvokeMajorGC(heap);
     // |Detach| will cause the buffer to be |Unregister|ed. Without
     // barriers and proper synchronization this will trigger a data race on
     // TSAN.
-    ab->Detach();
+    ab->Detach(v8::Local<v8::Value>()).Check();
   }
 }
 
 TEST(ArrayBuffer_NonLivePromotion) {
-  if (!FLAG_incremental_marking || FLAG_separate_gc_phases) return;
-  FLAG_concurrent_array_buffer_sweeping = false;
+  if (!v8_flags.incremental_marking || v8_flags.separate_gc_phases) return;
+  v8_flags.concurrent_array_buffer_sweeping = false;
   ManualGCScope manual_gc_scope;
   // The test verifies that the marking state is preserved when promoting
   // a buffer to old space.
@@ -219,37 +239,37 @@ TEST(ArrayBuffer_NonLivePromotion) {
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
   Heap* heap = reinterpret_cast<Isolate*>(isolate)->heap();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(
+      CcTest::heap());
 
-  JSArrayBuffer raw_ab;
   {
     v8::HandleScope handle_scope(isolate);
-    Handle<FixedArray> root =
+    DirectHandle<FixedArray> root =
         heap->isolate()->factory()->NewFixedArray(1, AllocationType::kOld);
     {
       v8::HandleScope new_handle_scope(isolate);
       Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 100);
-      Handle<JSArrayBuffer> buf = v8::Utils::OpenHandle(*ab);
+      DirectHandle<JSArrayBuffer> buf = v8::Utils::OpenDirectHandle(*ab);
       root->set(0, *buf);  // Buffer that should not be promoted as live.
     }
     heap::SimulateIncrementalMarking(heap, false);
-    CHECK(IsTracked(heap, JSArrayBuffer::cast(root->get(0))));
-    heap::GcAndSweep(heap, NEW_SPACE);
-    CHECK(IsTracked(heap, JSArrayBuffer::cast(root->get(0))));
-    heap::GcAndSweep(heap, NEW_SPACE);
-    CHECK(IsTracked(heap, JSArrayBuffer::cast(root->get(0))));
-    raw_ab = JSArrayBuffer::cast(root->get(0));
+    CHECK(IsTracked(heap, Cast<JSArrayBuffer>(root->get(0))));
+    heap::InvokeAtomicMinorGC(heap);
+    CHECK(IsTracked(heap, Cast<JSArrayBuffer>(root->get(0))));
+    heap::InvokeAtomicMinorGC(heap);
+    CHECK(IsTracked(heap, Cast<JSArrayBuffer>(root->get(0))));
+    ArrayBufferExtension* extension =
+        Cast<JSArrayBuffer>(root->get(0))->extension();
     root->set(0, ReadOnlyRoots(heap).undefined_value());
     heap::SimulateIncrementalMarking(heap, true);
-    // Prohibit page from being released.
-    Page::FromHeapObject(raw_ab)->MarkNeverEvacuate();
-    heap::GcAndSweep(heap, OLD_SPACE);
-    CHECK(!IsTracked(heap, raw_ab));
+    heap::InvokeAtomicMajorGC(heap);
+    CHECK(!IsTracked(heap, extension));
   }
 }
 
 TEST(ArrayBuffer_LivePromotion) {
-  if (!FLAG_incremental_marking || FLAG_separate_gc_phases) return;
-  FLAG_concurrent_array_buffer_sweeping = false;
+  if (!v8_flags.incremental_marking || v8_flags.separate_gc_phases) return;
+  v8_flags.concurrent_array_buffer_sweeping = false;
   ManualGCScope manual_gc_scope;
   // The test verifies that the marking state is preserved when promoting
   // a buffer to old space.
@@ -258,39 +278,42 @@ TEST(ArrayBuffer_LivePromotion) {
   v8::Isolate* isolate = env->GetIsolate();
   Heap* heap = reinterpret_cast<Isolate*>(isolate)->heap();
 
-  JSArrayBuffer raw_ab;
+  Tagged<JSArrayBuffer> raw_ab;
   {
     v8::HandleScope handle_scope(isolate);
-    Handle<FixedArray> root =
+    DirectHandle<FixedArray> root =
         heap->isolate()->factory()->NewFixedArray(1, AllocationType::kOld);
     {
       v8::HandleScope new_handle_scope(isolate);
       Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 100);
-      Handle<JSArrayBuffer> buf = v8::Utils::OpenHandle(*ab);
+      DirectHandle<JSArrayBuffer> buf = v8::Utils::OpenDirectHandle(*ab);
       root->set(0, *buf);  // Buffer that should be promoted as live.
     }
     // Store array in Global such that it is part of the root set when
     // starting incremental marking.
     v8::Global<Value> global_root(CcTest::isolate(),
-                                  Utils::ToLocal(Handle<Object>::cast(root)));
+                                  Utils::ToLocal(Cast<Object>(root)));
     heap::SimulateIncrementalMarking(heap, true);
-    CHECK(IsTracked(heap, JSArrayBuffer::cast(root->get(0))));
-    heap::GcAndSweep(heap, NEW_SPACE);
-    CHECK(IsTracked(heap, JSArrayBuffer::cast(root->get(0))));
-    heap::GcAndSweep(heap, NEW_SPACE);
-    CHECK(IsTracked(heap, JSArrayBuffer::cast(root->get(0))));
-    raw_ab = JSArrayBuffer::cast(root->get(0));
+    CHECK(IsTracked(heap, Cast<JSArrayBuffer>(root->get(0))));
+    heap::InvokeMinorGC(heap);
+    CHECK(IsTracked(heap, Cast<JSArrayBuffer>(root->get(0))));
+    heap::InvokeMinorGC(heap);
+    CHECK(IsTracked(heap, Cast<JSArrayBuffer>(root->get(0))));
+    raw_ab = Cast<JSArrayBuffer>(root->get(0));
     root->set(0, ReadOnlyRoots(heap).undefined_value());
     // Prohibit page from being released.
-    Page::FromHeapObject(raw_ab)->MarkNeverEvacuate();
-    heap::GcAndSweep(heap, OLD_SPACE);
+    MemoryChunk::FromHeapObject(raw_ab)->MarkNeverEvacuate();
+    heap::InvokeMajorGC(heap);
+    CHECK(!heap->array_buffer_sweeper()->sweeping_in_progress());
     CHECK(IsTracked(heap, raw_ab));
   }
 }
 
 TEST(ArrayBuffer_SemiSpaceCopyThenPagePromotion) {
-  if (!i::FLAG_incremental_marking) return;
-  FLAG_concurrent_array_buffer_sweeping = false;
+  if (!i::v8_flags.incremental_marking) return;
+  if (v8_flags.minor_ms) return;
+  v8_flags.concurrent_array_buffer_sweeping = false;
+  v8_flags.scavenger_precise_object_pinning = false;
   ManualGCScope manual_gc_scope;
   // The test verifies that the marking state is preserved across semispace
   // copy.
@@ -302,31 +325,35 @@ TEST(ArrayBuffer_SemiSpaceCopyThenPagePromotion) {
   heap::SealCurrentObjects(heap);
   {
     v8::HandleScope handle_scope(isolate);
-    Handle<FixedArray> root =
+    DirectHandle<FixedArray> root =
         heap->isolate()->factory()->NewFixedArray(1, AllocationType::kOld);
     {
       v8::HandleScope new_handle_scope(isolate);
       Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 100);
-      Handle<JSArrayBuffer> buf = v8::Utils::OpenHandle(*ab);
+      DirectHandle<JSArrayBuffer> buf = v8::Utils::OpenDirectHandle(*ab);
       root->set(0, *buf);  // Buffer that should be promoted as live.
-      Page::FromHeapObject(*buf)->MarkNeverEvacuate();
+      MemoryChunk::FromHeapObject(*buf)->MarkNeverEvacuate();
     }
-    std::vector<Handle<FixedArray>> handles;
+    DirectHandleVector<FixedArray> handles(isolate);
     // Make the whole page transition from new->old, getting the buffers
     // processed in the sweeper (relying on marking information) instead of
     // processing during newspace evacuation.
     heap::FillCurrentPage(heap->new_space(), &handles);
-    CHECK(IsTracked(heap, JSArrayBuffer::cast(root->get(0))));
-    heap::GcAndSweep(heap, NEW_SPACE);
+    CHECK(IsTracked(heap, Cast<JSArrayBuffer>(root->get(0))));
+    {
+      // CSS prevent semi space copying in Scavenger.
+      DisableConservativeStackScanningScopeForTesting no_stack_scanning(heap);
+      heap::InvokeAtomicMinorGC(heap);
+    }
     heap::SimulateIncrementalMarking(heap, true);
-    heap::GcAndSweep(heap, OLD_SPACE);
-    CHECK(IsTracked(heap, JSArrayBuffer::cast(root->get(0))));
+    heap::InvokeAtomicMajorGC(heap);
+    CHECK(IsTracked(heap, Cast<JSArrayBuffer>(root->get(0))));
   }
 }
 
 TEST(ArrayBuffer_PagePromotion) {
-  if (!i::FLAG_incremental_marking || i::FLAG_single_generation) return;
-  i::FLAG_concurrent_array_buffer_sweeping = false;
+  if (!i::v8_flags.incremental_marking || i::v8_flags.single_generation) return;
+  i::v8_flags.concurrent_array_buffer_sweeping = false;
 
   ManualGCScope manual_gc_scope;
   // The test verifies that the marking state is preserved across semispace
@@ -339,33 +366,33 @@ TEST(ArrayBuffer_PagePromotion) {
   heap::SealCurrentObjects(heap);
   {
     v8::HandleScope handle_scope(isolate);
-    Handle<FixedArray> root =
+    DirectHandle<FixedArray> root =
         heap->isolate()->factory()->NewFixedArray(1, AllocationType::kOld);
     ArrayBufferExtension* extension;
     {
       v8::HandleScope new_handle_scope(isolate);
       Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, 100);
-      Handle<JSArrayBuffer> buf = v8::Utils::OpenHandle(*ab);
+      DirectHandle<JSArrayBuffer> buf = v8::Utils::OpenDirectHandle(*ab);
       extension = buf->extension();
       root->set(0, *buf);  // Buffer that should be promoted as live.
     }
-    std::vector<Handle<FixedArray>> handles;
+    DirectHandleVector<FixedArray> handles(isolate);
     // Create live objects on page such that the whole page gets promoted
     heap::FillCurrentPage(heap->new_space(), &handles);
     CHECK(IsTrackedYoung(heap, extension));
     heap::SimulateIncrementalMarking(heap, true);
-    heap::GcAndSweep(heap, OLD_SPACE);
+    heap::InvokeAtomicMajorGC(heap);
     CHECK(IsTrackedOld(heap, extension));
   }
 }
 
 UNINITIALIZED_TEST(ArrayBuffer_SemiSpaceCopyMultipleTasks) {
-  if (FLAG_optimize_for_size || FLAG_single_generation) return;
+  if (v8_flags.optimize_for_size || v8_flags.single_generation) return;
   ManualGCScope manual_gc_scope;
   // Test allocates JSArrayBuffer on different pages before triggering a
   // full GC that performs the semispace copy. If parallelized, this test
   // ensures proper synchronization in TSAN configurations.
-  FLAG_min_semi_space_size = std::max(2 * Page::kPageSize / MB, 1);
+  v8_flags.min_semi_space_size = std::max(2 * PageMetadata::kPageSize / MB, 1);
   v8::Isolate::CreateParams create_params;
   create_params.array_buffer_allocator = CcTest::array_buffer_allocator();
   v8::Isolate* isolate = v8::Isolate::New(create_params);
@@ -377,29 +404,30 @@ UNINITIALIZED_TEST(ArrayBuffer_SemiSpaceCopyMultipleTasks) {
     Heap* heap = i_isolate->heap();
 
     // Ensure heap is in a clean state.
-    CcTest::CollectAllGarbage(i_isolate);
-    CcTest::CollectAllGarbage(i_isolate);
+    heap::InvokeMajorGC(heap);
+    heap::InvokeMajorGC(heap);
 
     Local<v8::ArrayBuffer> ab1 = v8::ArrayBuffer::New(isolate, 100);
-    Handle<JSArrayBuffer> buf1 = v8::Utils::OpenHandle(*ab1);
+    DirectHandle<JSArrayBuffer> buf1 = v8::Utils::OpenDirectHandle(*ab1);
     heap::FillCurrentPage(heap->new_space());
     Local<v8::ArrayBuffer> ab2 = v8::ArrayBuffer::New(isolate, 100);
-    Handle<JSArrayBuffer> buf2 = v8::Utils::OpenHandle(*ab2);
-    CHECK_NE(Page::FromHeapObject(*buf1), Page::FromHeapObject(*buf2));
-    heap::GcAndSweep(heap, OLD_SPACE);
+    DirectHandle<JSArrayBuffer> buf2 = v8::Utils::OpenDirectHandle(*ab2);
+    CHECK_NE(PageMetadata::FromHeapObject(*buf1),
+             PageMetadata::FromHeapObject(*buf2));
+    heap::InvokeAtomicMajorGC(heap);
   }
   isolate->Dispose();
 }
 
 TEST(ArrayBuffer_ExternalBackingStoreSizeIncreases) {
-  if (FLAG_single_generation) return;
+  if (v8_flags.single_generation) return;
   CcTest::InitializeVM();
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
   Heap* heap = reinterpret_cast<Isolate*>(isolate)->heap();
   ExternalBackingStoreType type = ExternalBackingStoreType::kArrayBuffer;
 
-  const Space* space = FLAG_incremental_marking
+  const Space* space = v8_flags.incremental_marking
                            ? static_cast<Space*>(heap->new_space())
                            : static_cast<Space*>(heap->old_space());
   const size_t backing_store_before = space->ExternalBackingStoreBytes(type);
@@ -414,8 +442,8 @@ TEST(ArrayBuffer_ExternalBackingStoreSizeIncreases) {
 }
 
 TEST(ArrayBuffer_ExternalBackingStoreSizeDecreases) {
-  if (FLAG_single_generation) return;
-  FLAG_concurrent_array_buffer_sweeping = false;
+  if (v8_flags.single_generation) return;
+  v8_flags.concurrent_array_buffer_sweeping = false;
   CcTest::InitializeVM();
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
@@ -430,17 +458,18 @@ TEST(ArrayBuffer_ExternalBackingStoreSizeDecreases) {
     Local<v8::ArrayBuffer> ab = v8::ArrayBuffer::New(isolate, kArraybufferSize);
     USE(ab);
   }
-  heap::GcAndSweep(heap, OLD_SPACE);
+  heap::InvokeAtomicMajorGC(heap);
   const size_t backing_store_after =
       heap->new_space()->ExternalBackingStoreBytes(type);
   CHECK_EQ(0, backing_store_after - backing_store_before);
 }
 
 TEST(ArrayBuffer_ExternalBackingStoreSizeIncreasesMarkCompact) {
-  if (!FLAG_compact) return;
+  if (!v8_flags.compact) return;
   ManualGCScope manual_gc_scope;
-  FLAG_manual_evacuation_candidates_selection = true;
-  FLAG_concurrent_array_buffer_sweeping = false;
+  heap::ManualEvacuationCandidatesSelectionScope
+      manual_evacuation_candidate_selection_scope(manual_gc_scope);
+  v8_flags.concurrent_array_buffer_sweeping = false;
   CcTest::InitializeVM();
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
@@ -448,6 +477,9 @@ TEST(ArrayBuffer_ExternalBackingStoreSizeIncreasesMarkCompact) {
   heap::AbandonCurrentlyFreeMemory(heap->old_space());
   ExternalBackingStoreType type = ExternalBackingStoreType::kArrayBuffer;
 
+  // We need to invoke GC without stack, otherwise some objects may survive.
+  DisableConservativeStackScanningScopeForTesting no_stack_scanning(heap);
+
   const size_t backing_store_before =
       heap->old_space()->ExternalBackingStoreBytes(type);
 
@@ -456,23 +488,22 @@ TEST(ArrayBuffer_ExternalBackingStoreSizeIncreasesMarkCompact) {
     v8::HandleScope handle_scope(isolate);
     Local<v8::ArrayBuffer> ab1 =
         v8::ArrayBuffer::New(isolate, kArraybufferSize);
-    Handle<JSArrayBuffer> buf1 = v8::Utils::OpenHandle(*ab1);
+    IndirectHandle<JSArrayBuffer> buf1 = v8::Utils::OpenIndirectHandle(*ab1);
     CHECK(IsTracked(heap, *buf1));
-    heap::GcAndSweep(heap, NEW_SPACE);
-    heap::GcAndSweep(heap, NEW_SPACE);
+    heap::InvokeAtomicMajorGC(heap);
 
-    Page* page_before_gc = Page::FromHeapObject(*buf1);
+    PageMetadata* page_before_gc = PageMetadata::FromHeapObject(*buf1);
     heap::ForceEvacuationCandidate(page_before_gc);
     CHECK(IsTracked(heap, *buf1));
 
-    CcTest::CollectAllGarbage();
+    heap::InvokeMajorGC(heap);
 
     const size_t backing_store_after =
         heap->old_space()->ExternalBackingStoreBytes(type);
     CHECK_EQ(kArraybufferSize, backing_store_after - backing_store_before);
   }
 
-  heap::GcAndSweep(heap, OLD_SPACE);
+  heap::InvokeAtomicMajorGC(heap);
   const size_t backing_store_after =
       heap->old_space()->ExternalBackingStoreBytes(type);
   CHECK_EQ(0, backing_store_after - backing_store_before);
