@@ -23,135 +23,142 @@ Handle<LoadHandler> CreateLoadHandlerForTest(
   Handle<LoadHandler> result = factory->NewLoadHandler(1, allocation);
   result->set_smi_handler(Smi::zero());
   result->set_validity_cell(Smi::zero());
-  result->set_data1(MaybeObject::FromSmi(Smi::zero()));
+  result->set_data1(Smi::zero());
   return result;
 }
 
 TEST(WeakReferencesBasic) {
+  ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(
+      CcTest::heap());
   HandleScope outer_scope(isolate);
 
-  Handle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
+  IndirectHandle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
 
-  if (!FLAG_single_generation) CHECK(Heap::InYoungGeneration(*lh));
+  if (!v8_flags.single_generation) CHECK(HeapLayout::InYoungGeneration(*lh));
 
-  MaybeObject code_object = lh->data1();
-  CHECK(code_object->IsSmi());
-  CcTest::CollectAllGarbage();
-  CHECK(!Heap::InYoungGeneration(*lh));
+  Tagged<MaybeObject> code_object = lh->data1();
+  CHECK(IsSmi(code_object));
+  heap::InvokeMajorGC(CcTest::heap());
+  CHECK(!HeapLayout::InYoungGeneration(*lh));
   CHECK_EQ(code_object, lh->data1());
 
   {
     HandleScope inner_scope(isolate);
 
     // Create a new Code.
-    Assembler assm(AssemblerOptions{});
+    Assembler assm(isolate->allocator(), AssemblerOptions{});
     assm.nop();  // supported on all architectures
     CodeDesc desc;
     assm.GetCode(isolate, &desc);
-    Handle<CodeT> code = ToCodeT(
-        Factory::CodeBuilder(isolate, desc, CodeKind::FOR_TESTING).Build(),
-        isolate);
-    CHECK(code->IsCodeT());
+    IndirectHandle<Code> code =
+        Factory::CodeBuilder(isolate, desc, CodeKind::FOR_TESTING).Build();
+    CHECK(IsCode(*code));
 
-    lh->set_data1(HeapObjectReference::Weak(*code));
-    HeapObject code_heap_object;
-    CHECK(lh->data1()->GetHeapObjectIfWeak(&code_heap_object));
-    CHECK_EQ(*code, code_heap_object);
+    // We cannot store the Code object itself into the tagged field as it will
+    // be located outside of the main pointer compression cage when the sandbox
+    // is enabled. So instead we use the Code's wrapper object.
+    lh->set_data1(MakeWeak(code->wrapper()));
+    Tagged<HeapObject> code_wrapper_heap_object;
+    CHECK(lh->data1().GetHeapObjectIfWeak(&code_wrapper_heap_object));
+    CHECK_EQ(code->wrapper(), code_wrapper_heap_object);
 
-    CcTest::CollectAllGarbage();
+    heap::InvokeMajorGC(CcTest::heap());
 
-    CHECK(lh->data1()->GetHeapObjectIfWeak(&code_heap_object));
-    CHECK_EQ(*code, code_heap_object);
+    CHECK(lh->data1().GetHeapObjectIfWeak(&code_wrapper_heap_object));
+    CHECK_EQ(code->wrapper(), code_wrapper_heap_object);
   }  // code will go out of scope.
 
-  CcTest::CollectAllGarbage();
-  CHECK(lh->data1()->IsCleared());
+  heap::InvokeMajorGC(CcTest::heap());
+  CHECK(lh->data1().IsCleared());
 }
 
 TEST(WeakReferencesOldToOld) {
-  if (FLAG_enable_third_party_heap) return;
   // Like WeakReferencesBasic, but the updated weak slot is in the old space,
   // and referring to an old space object.
   ManualGCScope manual_gc_scope;
-  FLAG_manual_evacuation_candidates_selection = true;
+  heap::ManualEvacuationCandidatesSelectionScope
+      manual_evacuation_candidate_selection_scope(manual_gc_scope);
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
 
   HandleScope outer_scope(isolate);
-  Handle<LoadHandler> lh =
+  DirectHandle<LoadHandler> lh =
       CreateLoadHandlerForTest(factory, AllocationType::kOld);
   CHECK(heap->InOldSpace(*lh));
 
   // Create a new FixedArray which the LoadHandler will point to.
-  Handle<FixedArray> fixed_array =
+  DirectHandle<FixedArray> fixed_array =
       factory->NewFixedArray(1, AllocationType::kOld);
   CHECK(heap->InOldSpace(*fixed_array));
-  lh->set_data1(HeapObjectReference::Weak(*fixed_array));
+  lh->set_data1(MakeWeak(*fixed_array));
 
-  Page* page_before_gc = Page::FromHeapObject(*fixed_array);
+  PageMetadata* page_before_gc = PageMetadata::FromHeapObject(*fixed_array);
   heap::ForceEvacuationCandidate(page_before_gc);
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(heap->InOldSpace(*fixed_array));
 
-  HeapObject heap_object;
-  CHECK(lh->data1()->GetHeapObjectIfWeak(&heap_object));
+  Tagged<HeapObject> heap_object;
+  CHECK(lh->data1().GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(heap_object, *fixed_array);
 }
 
 TEST(WeakReferencesOldToNew) {
   // Like WeakReferencesBasic, but the updated weak slot is in the old space,
   // and referring to an new space object.
-  if (FLAG_single_generation) return;
+  if (v8_flags.single_generation) return;
+  ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
 
   HandleScope outer_scope(isolate);
-  Handle<LoadHandler> lh =
+  DirectHandle<LoadHandler> lh =
       CreateLoadHandlerForTest(factory, AllocationType::kOld);
   CHECK(heap->InOldSpace(*lh));
 
   // Create a new FixedArray which the LoadHandler will point to.
-  Handle<FixedArray> fixed_array = factory->NewFixedArray(1);
-  CHECK(Heap::InYoungGeneration(*fixed_array));
-  lh->set_data1(HeapObjectReference::Weak(*fixed_array));
+  DirectHandle<FixedArray> fixed_array = factory->NewFixedArray(1);
+  CHECK(HeapLayout::InYoungGeneration(*fixed_array));
+  lh->set_data1(MakeWeak(*fixed_array));
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
 
-  HeapObject heap_object;
-  CHECK(lh->data1()->GetHeapObjectIfWeak(&heap_object));
+  Tagged<HeapObject> heap_object;
+  CHECK(lh->data1().GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(heap_object, *fixed_array);
 }
 
 TEST(WeakReferencesOldToNewScavenged) {
-  if (FLAG_single_generation) return;
+  if (v8_flags.single_generation) return;
   // Like WeakReferencesBasic, but the updated weak slot is in the old space,
   // and referring to an new space object, which is then scavenged.
+  ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
 
   HandleScope outer_scope(isolate);
-  Handle<LoadHandler> lh =
+  DirectHandle<LoadHandler> lh =
       CreateLoadHandlerForTest(factory, AllocationType::kOld);
   CHECK(heap->InOldSpace(*lh));
 
   // Create a new FixedArray which the LoadHandler will point to.
-  Handle<FixedArray> fixed_array = factory->NewFixedArray(1);
-  CHECK(Heap::InYoungGeneration(*fixed_array));
-  lh->set_data1(HeapObjectReference::Weak(*fixed_array));
+  DirectHandle<FixedArray> fixed_array = factory->NewFixedArray(1);
+  CHECK(HeapLayout::InYoungGeneration(*fixed_array));
+  lh->set_data1(MakeWeak(*fixed_array));
 
-  CcTest::CollectGarbage(NEW_SPACE);
+  heap::InvokeMinorGC(heap);
 
-  HeapObject heap_object;
-  CHECK(lh->data1()->GetHeapObjectIfWeak(&heap_object));
+  Tagged<HeapObject> heap_object;
+  CHECK(lh->data1().GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(heap_object, *fixed_array);
 }
 
@@ -159,25 +166,26 @@ TEST(WeakReferencesOldToCleared) {
   // Like WeakReferencesBasic, but the updated weak slot is in the old space,
   // and is cleared.
   ManualGCScope manual_gc_scope;
-  FLAG_manual_evacuation_candidates_selection = true;
+  heap::ManualEvacuationCandidatesSelectionScope
+      manual_evacuation_candidate_selection_scope(manual_gc_scope);
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
 
   HandleScope outer_scope(isolate);
-  Handle<LoadHandler> lh =
+  DirectHandle<LoadHandler> lh =
       CreateLoadHandlerForTest(factory, AllocationType::kOld);
   CHECK(heap->InOldSpace(*lh));
-  lh->set_data1(HeapObjectReference::ClearedValue(isolate));
+  lh->set_data1(ClearedValue(isolate));
 
-  CcTest::CollectAllGarbage();
-  CHECK(lh->data1()->IsCleared());
+  heap::InvokeMajorGC(heap);
+  CHECK(lh->data1().IsCleared());
 }
 
 TEST(ObjectMovesBeforeClearingWeakField) {
-  if (!FLAG_incremental_marking || FLAG_single_generation ||
-      FLAG_separate_gc_phases) {
+  if (!v8_flags.incremental_marking || v8_flags.single_generation ||
+      v8_flags.separate_gc_phases) {
     return;
   }
   ManualGCScope manual_gc_scope;
@@ -185,17 +193,18 @@ TEST(ObjectMovesBeforeClearingWeakField) {
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(heap);
 
   HandleScope outer_scope(isolate);
-  Handle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
+  IndirectHandle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
   CHECK(InCorrectGeneration(*lh));
-  LoadHandler lh_location = *lh;
+  Address lh_object_location = lh->address();
   {
     HandleScope inner_scope(isolate);
     // Create a new FixedArray which the LoadHandler will point to.
-    Handle<FixedArray> fixed_array = factory->NewFixedArray(1);
-    CHECK(Heap::InYoungGeneration(*fixed_array));
-    lh->set_data1(HeapObjectReference::Weak(*fixed_array));
+    IndirectHandle<FixedArray> fixed_array = factory->NewFixedArray(1);
+    CHECK(HeapLayout::InYoungGeneration(*fixed_array));
+    lh->set_data1(MakeWeak(*fixed_array));
     // inner_scope will go out of scope, so when marking the next time,
     // *fixed_array will stay white.
   }
@@ -205,18 +214,17 @@ TEST(ObjectMovesBeforeClearingWeakField) {
   SimulateIncrementalMarking(heap, true);
 
   // Scavenger will move *lh.
-  CcTest::CollectGarbage(NEW_SPACE);
-  LoadHandler new_lh_location = *lh;
-  CHECK_NE(lh_location, new_lh_location);
-  CHECK(lh->data1()->IsWeak());
+  heap::InvokeMinorGC(heap);
+  CHECK_NE(lh_object_location, lh.address());
+  CHECK(lh->data1().IsWeak());
 
   // Now we try to clear *lh.
-  CcTest::CollectAllGarbage();
-  CHECK(lh->data1()->IsCleared());
+  heap::InvokeMajorGC(heap);
+  CHECK(lh->data1().IsCleared());
 }
 
 TEST(ObjectWithWeakFieldDies) {
-  if (!FLAG_incremental_marking) {
+  if (!v8_flags.incremental_marking) {
     return;
   }
   ManualGCScope manual_gc_scope;
@@ -227,14 +235,14 @@ TEST(ObjectWithWeakFieldDies) {
 
   {
     HandleScope outer_scope(isolate);
-    Handle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
+    DirectHandle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
     CHECK(InCorrectGeneration(*lh));
     {
       HandleScope inner_scope(isolate);
       // Create a new FixedArray which the LoadHandler will point to.
-      Handle<FixedArray> fixed_array = factory->NewFixedArray(1);
+      DirectHandle<FixedArray> fixed_array = factory->NewFixedArray(1);
       CHECK(InCorrectGeneration(*fixed_array));
-      lh->set_data1(HeapObjectReference::Weak(*fixed_array));
+      lh->set_data1(MakeWeak(*fixed_array));
       // inner_scope will go out of scope, so when marking the next time,
       // *fixed_array will stay white.
     }
@@ -245,65 +253,62 @@ TEST(ObjectWithWeakFieldDies) {
   }  // outer_scope goes out of scope
 
   // lh will die
-  CcTest::CollectGarbage(NEW_SPACE);
+  heap::InvokeMinorGC(heap);
 
   // This used to crash when processing the dead weak reference.
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
 }
 
 TEST(ObjectWithWeakReferencePromoted) {
-  if (FLAG_single_generation) return;
+  if (v8_flags.single_generation) return;
+  ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
 
   HandleScope outer_scope(isolate);
-  Handle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
-  CHECK(Heap::InYoungGeneration(*lh));
+  DirectHandle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
+  CHECK(HeapLayout::InYoungGeneration(*lh));
 
   // Create a new FixedArray which the LoadHandler will point to.
-  Handle<FixedArray> fixed_array = factory->NewFixedArray(1);
-  CHECK(Heap::InYoungGeneration(*fixed_array));
-  lh->set_data1(HeapObjectReference::Weak(*fixed_array));
+  DirectHandle<FixedArray> fixed_array = factory->NewFixedArray(1);
+  CHECK(HeapLayout::InYoungGeneration(*fixed_array));
+  lh->set_data1(MakeWeak(*fixed_array));
 
-  CcTest::CollectGarbage(NEW_SPACE);
-  CcTest::CollectGarbage(NEW_SPACE);
+  heap::EmptyNewSpaceUsingGC(heap);
   CHECK(heap->InOldSpace(*lh));
   CHECK(heap->InOldSpace(*fixed_array));
 
-  HeapObject heap_object;
-  CHECK(lh->data1()->GetHeapObjectIfWeak(&heap_object));
+  Tagged<HeapObject> heap_object;
+  CHECK(lh->data1().GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(heap_object, *fixed_array);
 }
 
 TEST(ObjectWithClearedWeakReferencePromoted) {
-  if (FLAG_single_generation || FLAG_stress_incremental_marking) return;
+  if (v8_flags.single_generation || v8_flags.stress_incremental_marking) return;
+  ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
 
   HandleScope outer_scope(isolate);
-  Handle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
-  CHECK(Heap::InYoungGeneration(*lh));
+  DirectHandle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
+  CHECK(HeapLayout::InYoungGeneration(*lh));
 
-  lh->set_data1(HeapObjectReference::ClearedValue(isolate));
+  lh->set_data1(ClearedValue(isolate));
 
-  CcTest::CollectGarbage(NEW_SPACE);
-  CHECK(Heap::InYoungGeneration(*lh));
-  CHECK(lh->data1()->IsCleared());
-
-  CcTest::CollectGarbage(NEW_SPACE);
+  heap::EmptyNewSpaceUsingGC(heap);
   CHECK(heap->InOldSpace(*lh));
-  CHECK(lh->data1()->IsCleared());
+  CHECK(lh->data1().IsCleared());
 
-  CcTest::CollectAllGarbage();
-  CHECK(lh->data1()->IsCleared());
+  heap::InvokeMajorGC(heap);
+  CHECK(lh->data1().IsCleared());
 }
 
 TEST(WeakReferenceWriteBarrier) {
-  if (!FLAG_incremental_marking) {
+  if (!v8_flags.incremental_marking) {
     return;
   }
 
@@ -314,7 +319,7 @@ TEST(WeakReferenceWriteBarrier) {
   Heap* heap = isolate->heap();
 
   HandleScope outer_scope(isolate);
-  Handle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
+  DirectHandle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
   CHECK(InCorrectGeneration(*lh));
 
   v8::Global<Value> global_lh(CcTest::isolate(), Utils::ToLocal(lh));
@@ -323,22 +328,22 @@ TEST(WeakReferenceWriteBarrier) {
     HandleScope inner_scope(isolate);
 
     // Create a new FixedArray which the LoadHandler will point to.
-    Handle<FixedArray> fixed_array1 = factory->NewFixedArray(1);
+    DirectHandle<FixedArray> fixed_array1 = factory->NewFixedArray(1);
     CHECK(InCorrectGeneration(*fixed_array1));
-    lh->set_data1(HeapObjectReference::Weak(*fixed_array1));
+    lh->set_data1(MakeWeak(*fixed_array1));
 
     SimulateIncrementalMarking(heap, true);
 
-    Handle<FixedArray> fixed_array2 = factory->NewFixedArray(1);
+    DirectHandle<FixedArray> fixed_array2 = factory->NewFixedArray(1);
     CHECK(InCorrectGeneration(*fixed_array2));
     // This write will trigger the write barrier.
-    lh->set_data1(HeapObjectReference::Weak(*fixed_array2));
+    lh->set_data1(MakeWeak(*fixed_array2));
   }
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
 
   // Check that the write barrier treated the weak reference as strong.
-  CHECK(lh->data1()->IsWeak());
+  CHECK(lh->data1().IsWeak());
 }
 
 TEST(EmptyWeakArray) {
@@ -347,53 +352,54 @@ TEST(EmptyWeakArray) {
   Factory* factory = isolate->factory();
   HandleScope outer_scope(isolate);
 
-  Handle<WeakFixedArray> array = factory->empty_weak_fixed_array();
-  CHECK(array->IsWeakFixedArray());
-  CHECK(!array->IsFixedArray());
+  DirectHandle<WeakFixedArray> array = factory->empty_weak_fixed_array();
+  CHECK(IsWeakFixedArray(*array));
+  CHECK(!IsFixedArray(*array));
   CHECK_EQ(array->length(), 0);
 }
 
 TEST(WeakArraysBasic) {
-  if (FLAG_single_generation) return;
+  if (v8_flags.single_generation) return;
 
   ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(heap);
   HandleScope outer_scope(isolate);
 
   const int length = 4;
-  Handle<WeakFixedArray> array = factory->NewWeakFixedArray(length);
-  CHECK(array->IsWeakFixedArray());
-  CHECK(!array->IsFixedArray());
+  IndirectHandle<WeakFixedArray> array = factory->NewWeakFixedArray(length);
+  CHECK(IsWeakFixedArray(*array));
+  CHECK(!IsFixedArray(*array));
   CHECK_EQ(array->length(), length);
 
-  CHECK(Heap::InYoungGeneration(*array));
+  CHECK(HeapLayout::InYoungGeneration(*array));
 
   for (int i = 0; i < length; ++i) {
-    HeapObject heap_object;
-    CHECK(array->Get(i)->GetHeapObjectIfStrong(&heap_object));
+    Tagged<HeapObject> heap_object;
+    CHECK(array->get(i).GetHeapObjectIfStrong(&heap_object));
     CHECK_EQ(heap_object, ReadOnlyRoots(heap).undefined_value());
   }
 
-  Handle<HeapObject> saved;
+  IndirectHandle<HeapObject> saved;
   {
     HandleScope inner_scope(isolate);
-    Handle<FixedArray> index0 = factory->NewFixedArray(1);
+    IndirectHandle<FixedArray> index0 = factory->NewFixedArray(1);
     index0->set(0, Smi::FromInt(2016));
-    Handle<FixedArray> index1 = factory->NewFixedArray(1);
+    IndirectHandle<FixedArray> index1 = factory->NewFixedArray(1);
     index1->set(0, Smi::FromInt(2017));
 
-    Handle<FixedArray> index2 = factory->NewFixedArray(1);
+    IndirectHandle<FixedArray> index2 = factory->NewFixedArray(1);
     index2->set(0, Smi::FromInt(2018));
-    Handle<FixedArray> index3 = factory->NewFixedArray(1);
+    IndirectHandle<FixedArray> index3 = factory->NewFixedArray(1);
     index3->set(0, Smi::FromInt(2019));
 
-    array->Set(0, HeapObjectReference::Weak(*index0));
-    array->Set(1, HeapObjectReference::Weak(*index1));
-    array->Set(2, HeapObjectReference::Strong(*index2));
-    array->Set(3, HeapObjectReference::Weak(*index3));
+    array->set(0, MakeWeak(*index0));
+    array->set(1, MakeWeak(*index1));
+    array->set(2, *index2);
+    array->set(3, MakeWeak(*index3));
     saved = inner_scope.CloseAndEscape(index1);
   }  // inner_scope goes out of scope.
 
@@ -403,93 +409,93 @@ TEST(WeakArraysBasic) {
 
   // TODO(marja): update this when/if we do handle weak references in the new
   // space.
-  CcTest::CollectGarbage(NEW_SPACE);
-  HeapObject heap_object;
-  CHECK(array->Get(0)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2016);
-  CHECK(array->Get(1)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2017);
-  CHECK(array->Get(2)->GetHeapObjectIfStrong(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2018);
-  CHECK(array->Get(3)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2019);
+  heap::InvokeMinorGC(heap);
+  Tagged<HeapObject> heap_object;
+  CHECK(array->get(0).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2016);
+  CHECK(array->get(1).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2017);
+  CHECK(array->get(2).GetHeapObjectIfStrong(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2018);
+  CHECK(array->get(3).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2019);
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(heap->InOldSpace(*array));
-  CHECK(array->Get(0)->IsCleared());
-  CHECK(array->Get(1)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2017);
-  CHECK(array->Get(2)->GetHeapObjectIfStrong(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2018);
-  CHECK(array->Get(3)->IsCleared());
+  CHECK(array->get(0).IsCleared());
+  CHECK(array->get(1).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2017);
+  CHECK(array->get(2).GetHeapObjectIfStrong(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2018);
+  CHECK(array->get(3).IsCleared());
 }
 
 TEST(WeakArrayListBasic) {
-  if (FLAG_single_generation) return;
+  if (v8_flags.single_generation) return;
 
   ManualGCScope manual_gc_scope;
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(heap);
   HandleScope outer_scope(isolate);
 
   Handle<WeakArrayList> array(ReadOnlyRoots(heap).empty_weak_array_list(),
                               isolate);
-  CHECK(array->IsWeakArrayList());
-  CHECK(!array->IsFixedArray());
-  CHECK(!array->IsWeakFixedArray());
+  CHECK(IsWeakArrayList(*array));
+  CHECK(!IsFixedArray(*array));
+  CHECK(!IsWeakFixedArray(*array));
   CHECK_EQ(array->length(), 0);
 
-  Handle<FixedArray> index2 = factory->NewFixedArray(1);
+  DirectHandle<FixedArray> index2 = factory->NewFixedArray(1);
   index2->set(0, Smi::FromInt(2017));
 
-  Handle<HeapObject> saved;
   {
     HandleScope inner_scope(isolate);
-    Handle<FixedArray> index0 = factory->NewFixedArray(1);
+    DirectHandle<FixedArray> index0 = factory->NewFixedArray(1);
     index0->set(0, Smi::FromInt(2016));
-    Handle<FixedArray> index4 = factory->NewFixedArray(1);
+    DirectHandle<FixedArray> index4 = factory->NewFixedArray(1);
     index4->set(0, Smi::FromInt(2018));
-    Handle<FixedArray> index6 = factory->NewFixedArray(1);
+    DirectHandle<FixedArray> index6 = factory->NewFixedArray(1);
     index6->set(0, Smi::FromInt(2019));
 
     array = WeakArrayList::AddToEnd(isolate, array,
-                                    MaybeObjectHandle::Weak(index0));
+                                    MaybeObjectDirectHandle::Weak(index0));
     array = WeakArrayList::AddToEnd(
-        isolate, array, MaybeObjectHandle(Smi::FromInt(1), isolate));
+        isolate, array, MaybeObjectDirectHandle(Smi::FromInt(1), isolate));
     CHECK_EQ(array->length(), 2);
 
     array = WeakArrayList::AddToEnd(isolate, array,
-                                    MaybeObjectHandle::Weak(index2));
+                                    MaybeObjectDirectHandle::Weak(index2));
     array = WeakArrayList::AddToEnd(
-        isolate, array, MaybeObjectHandle(Smi::FromInt(3), isolate));
+        isolate, array, MaybeObjectDirectHandle(Smi::FromInt(3), isolate));
     CHECK_EQ(array->length(), 4);
 
     array = WeakArrayList::AddToEnd(isolate, array,
-                                    MaybeObjectHandle::Weak(index4));
+                                    MaybeObjectDirectHandle::Weak(index4));
     array = WeakArrayList::AddToEnd(
-        isolate, array, MaybeObjectHandle(Smi::FromInt(5), isolate));
+        isolate, array, MaybeObjectDirectHandle(Smi::FromInt(5), isolate));
     CHECK_EQ(array->length(), 6);
 
     array = WeakArrayList::AddToEnd(isolate, array,
-                                    MaybeObjectHandle::Weak(index6));
+                                    MaybeObjectDirectHandle::Weak(index6));
     array = WeakArrayList::AddToEnd(
-        isolate, array, MaybeObjectHandle(Smi::FromInt(7), isolate));
+        isolate, array, MaybeObjectDirectHandle(Smi::FromInt(7), isolate));
     CHECK_EQ(array->length(), 8);
 
     CHECK(InCorrectGeneration(*array));
 
-    CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*index0));
-    CHECK_EQ(array->Get(1).ToSmi().value(), 1);
+    CHECK_EQ(array->get(0), MakeWeak(*index0));
+    CHECK_EQ(array->get(1).ToSmi().value(), 1);
 
-    CHECK_EQ(array->Get(2), HeapObjectReference::Weak(*index2));
-    CHECK_EQ(array->Get(3).ToSmi().value(), 3);
+    CHECK_EQ(array->get(2), MakeWeak(*index2));
+    CHECK_EQ(array->get(3).ToSmi().value(), 3);
 
-    CHECK_EQ(array->Get(4), HeapObjectReference::Weak(*index4));
-    CHECK_EQ(array->Get(5).ToSmi().value(), 5);
+    CHECK_EQ(array->get(4), MakeWeak(*index4));
+    CHECK_EQ(array->get(5).ToSmi().value(), 5);
 
-    CHECK_EQ(array->Get(6), HeapObjectReference::Weak(*index6));
+    CHECK_EQ(array->get(6), MakeWeak(*index6));
     array = inner_scope.CloseAndEscape(array);
   }  // inner_scope goes out of scope.
 
@@ -499,40 +505,40 @@ TEST(WeakArrayListBasic) {
 
   // TODO(marja): update this when/if we do handle weak references in the new
   // space.
-  CcTest::CollectGarbage(NEW_SPACE);
-  HeapObject heap_object;
+  heap::InvokeMinorGC(heap);
+  Tagged<HeapObject> heap_object;
   CHECK_EQ(array->length(), 8);
-  CHECK(array->Get(0)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2016);
-  CHECK_EQ(array->Get(1).ToSmi().value(), 1);
+  CHECK(array->get(0).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2016);
+  CHECK_EQ(array->get(1).ToSmi().value(), 1);
 
-  CHECK(array->Get(2)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2017);
-  CHECK_EQ(array->Get(3).ToSmi().value(), 3);
+  CHECK(array->get(2).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2017);
+  CHECK_EQ(array->get(3).ToSmi().value(), 3);
 
-  CHECK(array->Get(4)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2018);
-  CHECK_EQ(array->Get(5).ToSmi().value(), 5);
+  CHECK(array->get(4).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2018);
+  CHECK_EQ(array->get(5).ToSmi().value(), 5);
 
-  CHECK(array->Get(6)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2019);
-  CHECK_EQ(array->Get(7).ToSmi().value(), 7);
+  CHECK(array->get(6).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2019);
+  CHECK_EQ(array->get(7).ToSmi().value(), 7);
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(heap->InOldSpace(*array));
   CHECK_EQ(array->length(), 8);
-  CHECK(array->Get(0)->IsCleared());
-  CHECK_EQ(array->Get(1).ToSmi().value(), 1);
+  CHECK(array->get(0).IsCleared());
+  CHECK_EQ(array->get(1).ToSmi().value(), 1);
 
-  CHECK(array->Get(2)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2017);
-  CHECK_EQ(array->Get(3).ToSmi().value(), 3);
+  CHECK(array->get(2).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Cast<Smi>(Cast<FixedArray>(heap_object)->get(0)).value(), 2017);
+  CHECK_EQ(array->get(3).ToSmi().value(), 3);
 
-  CHECK(array->Get(4)->IsCleared());
-  CHECK_EQ(array->Get(5).ToSmi().value(), 5);
+  CHECK(array->get(4).IsCleared());
+  CHECK_EQ(array->get(5).ToSmi().value(), 5);
 
-  CHECK(array->Get(6)->IsCleared());
-  CHECK_EQ(array->Get(7).ToSmi().value(), 7);
+  CHECK(array->get(6).IsCleared());
+  CHECK_EQ(array->get(7).ToSmi().value(), 7);
 }
 
 TEST(WeakArrayListRemove) {
@@ -546,48 +552,89 @@ TEST(WeakArrayListRemove) {
   Handle<WeakArrayList> array(ReadOnlyRoots(heap).empty_weak_array_list(),
                               isolate);
 
-  Handle<FixedArray> elem0 = factory->NewFixedArray(1);
-  Handle<FixedArray> elem1 = factory->NewFixedArray(1);
-  Handle<FixedArray> elem2 = factory->NewFixedArray(1);
+  DirectHandle<FixedArray> elem0 = factory->NewFixedArray(1);
+  DirectHandle<FixedArray> elem1 = factory->NewFixedArray(1);
+  DirectHandle<FixedArray> elem2 = factory->NewFixedArray(1);
 
-  array =
-      WeakArrayList::AddToEnd(isolate, array, MaybeObjectHandle::Weak(elem0));
-  array =
-      WeakArrayList::AddToEnd(isolate, array, MaybeObjectHandle::Weak(elem1));
-  array =
-      WeakArrayList::AddToEnd(isolate, array, MaybeObjectHandle::Weak(elem2));
+  array = WeakArrayList::AddToEnd(isolate, array,
+                                  MaybeObjectDirectHandle::Weak(elem0));
+  array = WeakArrayList::AddToEnd(isolate, array,
+                                  MaybeObjectDirectHandle::Weak(elem1));
+  array = WeakArrayList::AddToEnd(isolate, array,
+                                  MaybeObjectDirectHandle::Weak(elem2));
 
   CHECK_EQ(array->length(), 3);
-  CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*elem0));
-  CHECK_EQ(array->Get(1), HeapObjectReference::Weak(*elem1));
-  CHECK_EQ(array->Get(2), HeapObjectReference::Weak(*elem2));
+  CHECK_EQ(array->get(0), MakeWeak(*elem0));
+  CHECK_EQ(array->get(1), MakeWeak(*elem1));
+  CHECK_EQ(array->get(2), MakeWeak(*elem2));
 
-  CHECK(array->RemoveOne(MaybeObjectHandle::Weak(elem1)));
+  CHECK(array->RemoveOne(MaybeObjectDirectHandle::Weak(elem1)));
 
   CHECK_EQ(array->length(), 2);
-  CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*elem0));
-  CHECK_EQ(array->Get(1), HeapObjectReference::Weak(*elem2));
+  CHECK_EQ(array->get(0), MakeWeak(*elem0));
+  CHECK_EQ(array->get(1), MakeWeak(*elem2));
 
-  CHECK(!array->RemoveOne(MaybeObjectHandle::Weak(elem1)));
+  CHECK(!array->RemoveOne(MaybeObjectDirectHandle::Weak(elem1)));
 
   CHECK_EQ(array->length(), 2);
-  CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*elem0));
-  CHECK_EQ(array->Get(1), HeapObjectReference::Weak(*elem2));
+  CHECK_EQ(array->get(0), MakeWeak(*elem0));
+  CHECK_EQ(array->get(1), MakeWeak(*elem2));
 
-  CHECK(array->RemoveOne(MaybeObjectHandle::Weak(elem0)));
+  CHECK(array->RemoveOne(MaybeObjectDirectHandle::Weak(elem0)));
 
   CHECK_EQ(array->length(), 1);
-  CHECK_EQ(array->Get(0), HeapObjectReference::Weak(*elem2));
+  CHECK_EQ(array->get(0), MakeWeak(*elem2));
 
-  CHECK(array->RemoveOne(MaybeObjectHandle::Weak(elem2)));
+  CHECK(array->RemoveOne(MaybeObjectDirectHandle::Weak(elem2)));
 
   CHECK_EQ(array->length(), 0);
 }
 
+TEST(ProtectedWeakFixedArray) {
+  ManualGCScope manual_gc_scope;
+  CcTest::InitializeVM();
+  Isolate* isolate = CcTest::i_isolate();
+  Factory* factory = isolate->factory();
+  Heap* heap = isolate->heap();
+  HandleScope handle_scope(isolate);
+
+  IndirectHandle<ProtectedWeakFixedArray> array =
+      factory->NewProtectedWeakFixedArray(5);
+
+  IndirectHandle<TrustedFixedArray> elem1 = factory->NewTrustedFixedArray(1);
+  IndirectHandle<TrustedFixedArray> elem3 = factory->NewTrustedFixedArray(1);
+  array->set(1, MakeWeak(*elem1));
+  array->set(3, MakeWeak(*elem3));
+
+  {
+    HandleScope inner_scope(isolate);
+    DirectHandle<TrustedFixedArray> elem0 = factory->NewTrustedFixedArray(1);
+    DirectHandle<TrustedFixedArray> elem2 = factory->NewTrustedFixedArray(1);
+    DirectHandle<TrustedFixedArray> elem4 = factory->NewTrustedFixedArray(1);
+    array->set(0, MakeWeak(*elem0));
+    array->set(2, MakeWeak(*elem2));
+    array->set(4, MakeWeak(*elem4));
+    heap::InvokeMajorGC(heap);
+    CHECK_EQ(array->get(0).GetHeapObjectAssumeWeak(), *elem0);
+    CHECK_EQ(array->get(1).GetHeapObjectAssumeWeak(), *elem1);
+    CHECK_EQ(array->get(2).GetHeapObjectAssumeWeak(), *elem2);
+    CHECK_EQ(array->get(3).GetHeapObjectAssumeWeak(), *elem3);
+    CHECK_EQ(array->get(4).GetHeapObjectAssumeWeak(), *elem4);
+  }
+
+  DisableConservativeStackScanningScopeForTesting no_stack_scanning(heap);
+  heap::InvokeMajorGC(heap);
+  CHECK(array->get(0).IsCleared());
+  CHECK_EQ(array->get(1).GetHeapObjectAssumeWeak(), *elem1);
+  CHECK(array->get(2).IsCleared());
+  CHECK_EQ(array->get(3).GetHeapObjectAssumeWeak(), *elem3);
+  CHECK(array->get(4).IsCleared());
+}
+
 TEST(Regress7768) {
-  i::FLAG_allow_natives_syntax = true;
-  i::FLAG_turbo_inlining = false;
-  if (!FLAG_incremental_marking) {
+  i::v8_flags.allow_natives_syntax = true;
+  i::v8_flags.turbo_inlining = false;
+  if (!v8_flags.incremental_marking) {
     return;
   }
   ManualGCScope manual_gc_scope;
@@ -618,14 +665,14 @@ TEST(Regress7768) {
   CompileRun("%DeoptimizeFunction(myfunc);");
 
   // The object pointed to by the weak reference won't be scavenged.
-  CcTest::CollectGarbage(NEW_SPACE);
+  heap::InvokeMinorGC(heap);
 
   // Make sure the memory where it's stored is invalidated, so that we'll crash
   // if we try to access it.
-  HeapTester::UncommitFromSpace(heap);
+  HeapTester::UncommitUnusedMemory(heap);
 
   // This used to crash when processing the dead weak reference.
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
 }
 
 TEST(PrototypeUsersBasic) {
@@ -642,7 +689,8 @@ TEST(PrototypeUsersBasic) {
   // Add some objects into the array.
   int index = -1;
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    DirectHandle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
     CHECK_EQ(array->length(), index + 1);
   }
@@ -655,7 +703,8 @@ TEST(PrototypeUsersBasic) {
   int last_index = index;
   int old_capacity = array->capacity();
   while (!array->IsFull()) {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    DirectHandle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
     CHECK_EQ(index, last_index + 1);
     CHECK_EQ(array->length(), index + 1);
@@ -664,14 +713,16 @@ TEST(PrototypeUsersBasic) {
 
   // The next addition will fill the empty slot.
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    DirectHandle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
   }
   CHECK_EQ(index, empty_index);
 
   // The next addition will make the arrow grow again.
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    DirectHandle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
     CHECK_EQ(array->length(), index + 1);
     last_index = index;
@@ -687,7 +738,8 @@ TEST(PrototypeUsersBasic) {
   // Fill the array (still adding to the end)
   old_capacity = array->capacity();
   while (!array->IsFull()) {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    DirectHandle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
     CHECK_EQ(index, last_index + 1);
     CHECK_EQ(array->length(), index + 1);
@@ -696,13 +748,15 @@ TEST(PrototypeUsersBasic) {
 
   // Make sure we use the empty slots in (reverse) order.
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    DirectHandle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
   }
   CHECK_EQ(index, empty_index2);
 
   {
-    Handle<Map> map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    DirectHandle<Map> map = factory->NewContextfulMapForCurrentContext(
+        JS_OBJECT_TYPE, JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, map, &index);
   }
   CHECK_EQ(index, empty_index1);
@@ -710,9 +764,9 @@ TEST(PrototypeUsersBasic) {
 
 namespace {
 
-HeapObject saved_heap_object;
+Tagged<HeapObject> saved_heap_object;
 
-static void TestCompactCallback(HeapObject value, int old_index,
+static void TestCompactCallback(Tagged<HeapObject> value, int old_index,
                                 int new_index) {
   saved_heap_object = value;
   CHECK_EQ(old_index, 2);
@@ -728,6 +782,7 @@ TEST(PrototypeUsersCompacted) {
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
   Heap* heap = isolate->heap();
+  i::DisableConservativeStackScanningScopeForTesting no_stack_scanning(heap);
   HandleScope outer_scope(isolate);
 
   Handle<WeakArrayList> array(ReadOnlyRoots(heap).empty_weak_array_list(),
@@ -735,17 +790,20 @@ TEST(PrototypeUsersCompacted) {
 
   // Add some objects into the array.
   int index = -1;
-  Handle<Map> map_cleared_by_user =
-      factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+  DirectHandle<Map> map_cleared_by_user =
+      factory->NewContextfulMapForCurrentContext(JS_OBJECT_TYPE,
+                                                 JSObject::kHeaderSize);
   array = PrototypeUsers::Add(isolate, array, map_cleared_by_user, &index);
   CHECK_EQ(index, 1);
-  Handle<Map> live_map = factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+  DirectHandle<Map> live_map = factory->NewContextfulMapForCurrentContext(
+      JS_OBJECT_TYPE, JSObject::kHeaderSize);
   array = PrototypeUsers::Add(isolate, array, live_map, &index);
   CHECK_EQ(index, 2);
   {
     HandleScope inner_scope(isolate);
-    Handle<Map> soon_dead_map =
-        factory->NewMap(JS_OBJECT_TYPE, JSObject::kHeaderSize);
+    DirectHandle<Map> soon_dead_map =
+        factory->NewContextfulMapForCurrentContext(JS_OBJECT_TYPE,
+                                                   JSObject::kHeaderSize);
     array = PrototypeUsers::Add(isolate, array, soon_dead_map, &index);
     CHECK_EQ(index, 3);
 
@@ -753,13 +811,13 @@ TEST(PrototypeUsersCompacted) {
   }
 
   PrototypeUsers::MarkSlotEmpty(*array, 1);
-  CcTest::CollectAllGarbage();
-  CHECK(array->Get(3)->IsCleared());
+  heap::InvokeMajorGC(heap);
+  CHECK(array->get(3).IsCleared());
 
   CHECK_EQ(array->length(), 3 + PrototypeUsers::kFirstIndex);
-  WeakArrayList new_array =
+  Tagged<WeakArrayList> new_array =
       PrototypeUsers::Compact(array, heap, TestCompactCallback);
-  CHECK_EQ(new_array.length(), 1 + PrototypeUsers::kFirstIndex);
+  CHECK_EQ(new_array->length(), 1 + PrototypeUsers::kFirstIndex);
   CHECK_EQ(saved_heap_object, *live_map);
 }
 
