@@ -168,11 +168,11 @@ TEST(Unwind_BuiltinPCInMiddle_Success_CodePagesAPI) {
   register_state.fp = stack;
 
   // Put the current PC inside of a valid builtin.
-  Code builtin = FromCodeT(i_isolate->builtins()->code(Builtin::kStringEqual));
+  Tagged<Code> builtin = *BUILTIN_CODE(i_isolate, StringEqual);
   const uintptr_t offset = 40;
-  CHECK_LT(offset, builtin.InstructionSize());
+  CHECK_LT(offset, builtin->instruction_size());
   register_state.pc =
-      reinterpret_cast<void*>(builtin.InstructionStart() + offset);
+      reinterpret_cast<void*>(builtin->instruction_start() + offset);
 
   bool unwound = v8::Unwinder::TryUnwindV8Frames(
       entry_stubs, pages_length, code_pages, &register_state, stack_base);
@@ -225,8 +225,8 @@ TEST(Unwind_BuiltinPCAtStart_Success_CodePagesAPI) {
 
   // Put the current PC at the start of a valid builtin, so that we are setting
   // up the frame.
-  Code builtin = FromCodeT(i_isolate->builtins()->code(Builtin::kStringEqual));
-  register_state.pc = reinterpret_cast<void*>(builtin.InstructionStart());
+  Tagged<Code> builtin = *BUILTIN_CODE(i_isolate, StringEqual);
+  register_state.pc = reinterpret_cast<void*>(builtin->instruction_start());
 
   bool unwound = v8::Unwinder::TryUnwindV8Frames(
       entry_stubs, pages_length, code_pages, &register_state, stack_base);
@@ -253,10 +253,10 @@ const char* foo_source = R"(
 
 bool PagesContainsAddress(size_t length, MemoryRange* pages,
                           Address search_address) {
-  byte* addr = reinterpret_cast<byte*>(search_address);
+  uint8_t* addr = reinterpret_cast<uint8_t*>(search_address);
   auto it = std::find_if(pages, pages + length, [addr](const MemoryRange& r) {
-    const byte* page_start = reinterpret_cast<const byte*>(r.start);
-    const byte* page_end = page_start + r.length_in_bytes;
+    const uint8_t* page_start = reinterpret_cast<const uint8_t*>(r.start);
+    const uint8_t* page_end = page_start + r.length_in_bytes;
     return addr >= page_start && addr < page_end;
   });
   return it != pages + length;
@@ -265,7 +265,7 @@ bool PagesContainsAddress(size_t length, MemoryRange* pages,
 // Check that we can unwind when the pc is within an optimized code object on
 // the V8 heap.
 TEST(Unwind_CodeObjectPCInMiddle_Success_CodePagesAPI) {
-  FLAG_allow_natives_syntax = true;
+  v8_flags.allow_natives_syntax = true;
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
   Isolate* i_isolate = reinterpret_cast<Isolate*>(isolate);
@@ -292,22 +292,21 @@ TEST(Unwind_CodeObjectPCInMiddle_Success_CodePagesAPI) {
   CompileRun(foo_source);
   v8::Local<v8::Function> local_foo = v8::Local<v8::Function>::Cast(
       env.local()->Global()->Get(env.local(), v8_str("foo")).ToLocalChecked());
-  Handle<JSFunction> foo =
-      Handle<JSFunction>::cast(v8::Utils::OpenHandle(*local_foo));
+  DirectHandle<JSFunction> foo =
+      Cast<JSFunction>(v8::Utils::OpenDirectHandle(*local_foo));
 
   // Put the current PC inside of the created code object.
-  AbstractCode abstract_code = foo->abstract_code(i_isolate);
-  // We don't produce optimized code when run with --no-opt.
-  if (!abstract_code.IsCode() && FLAG_opt == false) return;
-  CHECK(abstract_code.IsCode());
+  Tagged<Code> code = foo->code(i_isolate);
+  // We don't produce optimized code when run with --no-turbofan and
+  // --no-maglev.
+  if (!code->is_optimized_code()) return;
 
-  Code code = abstract_code.GetCode();
   // We don't want the offset too early or it could be the `push rbp`
   // instruction (which is not at the start of generated code, because the lazy
   // deopt check happens before frame setup).
-  const uintptr_t offset = code.InstructionSize() - 20;
-  CHECK_LT(offset, code.InstructionSize());
-  Address pc = code.InstructionStart() + offset;
+  const uintptr_t offset = code->instruction_size() - 20;
+  CHECK_LT(offset, code->instruction_size());
+  Address pc = code->instruction_start() + offset;
   register_state.pc = reinterpret_cast<void*>(pc);
 
   // Get code pages from the API now that the code obejct exists and check that
@@ -456,8 +455,8 @@ TEST(Unwind_JSEntry_Fail_CodePagesAPI) {
   CHECK_LE(pages_length, arraysize(code_pages));
   RegisterState register_state;
 
-  Code js_entry = FromCodeT(i_isolate->builtins()->code(Builtin::kJSEntry));
-  byte* start = reinterpret_cast<byte*>(js_entry.InstructionStart());
+  Tagged<Code> js_entry = *BUILTIN_CODE(i_isolate, JSEntry);
+  uint8_t* start = reinterpret_cast<uint8_t*>(js_entry->instruction_start());
   register_state.pc = start + 10;
 
   bool unwound = v8::Unwinder::TryUnwindV8Frames(
@@ -594,7 +593,7 @@ TEST(PCIsInV8_ValidStateNullPC_Fail_CodePagesAPI) {
 }
 
 void TestRangeBoundaries(size_t pages_length, MemoryRange* code_pages,
-                         byte* range_start, size_t range_length) {
+                         uint8_t* range_start, size_t range_length) {
   void* pc = range_start - 1;
   CHECK(!v8::Unwinder::PCIsInV8(pages_length, code_pages, pc));
   pc = range_start;
@@ -619,8 +618,8 @@ TEST(PCIsInV8_InAllCodePages_CodePagesAPI) {
   CHECK_LE(pages_length, arraysize(code_pages));
 
   for (size_t i = 0; i < pages_length; i++) {
-    byte* range_start =
-        const_cast<byte*>(reinterpret_cast<const byte*>(code_pages[i].start));
+    uint8_t* range_start = const_cast<uint8_t*>(
+        reinterpret_cast<const uint8_t*>(code_pages[i].start));
     size_t range_length = code_pages[i].length_in_bytes;
     TestRangeBoundaries(pages_length, code_pages, range_start, range_length);
   }
@@ -638,9 +637,9 @@ TEST(PCIsInV8_InJSEntryRange_CodePagesAPI) {
       isolate->CopyCodePages(arraysize(code_pages), code_pages);
   CHECK_LE(pages_length, arraysize(code_pages));
 
-  Code js_entry = FromCodeT(i_isolate->builtins()->code(Builtin::kJSEntry));
-  byte* start = reinterpret_cast<byte*>(js_entry.InstructionStart());
-  size_t length = js_entry.InstructionSize();
+  Tagged<Code> js_entry = *BUILTIN_CODE(i_isolate, JSEntry);
+  uint8_t* start = reinterpret_cast<uint8_t*>(js_entry->instruction_start());
+  size_t length = js_entry->instruction_size();
 
   void* pc = start;
   CHECK(v8::Unwinder::PCIsInV8(pages_length, code_pages, pc));
@@ -653,16 +652,16 @@ TEST(PCIsInV8_InJSEntryRange_CodePagesAPI) {
 // Large code objects can be allocated in large object space. Check that this is
 // inside the CodeRange.
 TEST(PCIsInV8_LargeCodeObject_CodePagesAPI) {
-  FLAG_allow_natives_syntax = true;
+  v8_flags.allow_natives_syntax = true;
   LocalContext env;
   v8::Isolate* isolate = env->GetIsolate();
   Isolate* i_isolate = reinterpret_cast<Isolate*>(isolate);
   HandleScope scope(i_isolate);
 
   // Create a big function that ends up in CODE_LO_SPACE.
-  const int instruction_size = Page::kPageSize + 1;
+  const int instruction_size = PageMetadata::kPageSize + 1;
   CHECK_GT(instruction_size, MemoryChunkLayout::MaxRegularCodeObjectSize());
-  std::unique_ptr<byte[]> instructions(new byte[instruction_size]);
+  std::unique_ptr<uint8_t[]> instructions(new uint8_t[instruction_size]);
 
   CodeDesc desc;
   desc.buffer = instructions.get();
@@ -673,11 +672,12 @@ TEST(PCIsInV8_LargeCodeObject_CodePagesAPI) {
   desc.unwinding_info = nullptr;
   desc.unwinding_info_size = 0;
   desc.origin = nullptr;
-  Handle<Code> foo_code =
-      Factory::CodeBuilder(i_isolate, desc, CodeKind::WASM_FUNCTION).Build();
+  DirectHandle<Code> foo_code =
+      Factory::CodeBuilder(i_isolate, desc, CodeKind::FOR_TESTING).Build();
 
-  CHECK(i_isolate->heap()->InSpace(*foo_code, CODE_LO_SPACE));
-  byte* start = reinterpret_cast<byte*>(foo_code->InstructionStart());
+  CHECK(i_isolate->heap()->InSpace(foo_code->instruction_stream(),
+                                   CODE_LO_SPACE));
+  uint8_t* start = reinterpret_cast<uint8_t*>(foo_code->instruction_start());
 
   MemoryRange code_pages[v8::Isolate::kMinCodePagesBufferSize];
   size_t pages_length =
@@ -708,7 +708,8 @@ class UnwinderTestHelper {
   ~UnwinderTestHelper() { instance_ = nullptr; }
 
  private:
-  static void TryUnwind(const v8::FunctionCallbackInfo<v8::Value>& args) {
+  static void TryUnwind(const v8::FunctionCallbackInfo<v8::Value>& info) {
+    CHECK(i::ValidateCallbackInfo(info));
     instance_->DoTryUnwind();
   }
 
@@ -746,7 +747,7 @@ class UnwinderTestHelper {
 UnwinderTestHelper* UnwinderTestHelper::instance_;
 
 TEST(Unwind_TwoNestedFunctions_CodePagesAPI) {
-  i::FLAG_allow_natives_syntax = true;
+  i::v8_flags.allow_natives_syntax = true;
   const char* test_script =
       "function test_unwinder_api_inner() {"
       "  TryUnwind();"
