@@ -63,7 +63,7 @@ void TestModuleException(Zone* zone, WasmModuleBuilder* builder) {
   v8::TryCatch try_catch(reinterpret_cast<v8::Isolate*>(isolate));
   testing::CompileAndRunWasmModule(isolate, buffer.begin(), buffer.end());
   CHECK(try_catch.HasCaught());
-  isolate->clear_pending_exception();
+  isolate->clear_exception();
 }
 
 void ExportAsMain(WasmFunctionBuilder* f) {
@@ -88,7 +88,7 @@ TEST(Run_WasmModule_Return114) {
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_I32V_2(kReturnValue)};
+    uint8_t code[] = {WASM_I32V_2(kReturnValue)};
     EMIT_CODE_WITH_END(f, code);
     TestModule(&zone, builder, kReturnValue);
   }
@@ -96,7 +96,7 @@ TEST(Run_WasmModule_Return114) {
 }
 
 TEST(Run_WasmModule_CompilationHintsLazy) {
-  if (!FLAG_wasm_tier_up || !FLAG_liftoff) return;
+  if (!v8_flags.wasm_tier_up || !v8_flags.liftoff) return;
   {
     EXPERIMENTAL_FLAG_SCOPE(compilation_hints);
 
@@ -109,7 +109,7 @@ TEST(Run_WasmModule_CompilationHintsLazy) {
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_I32V_2(kReturnValue)};
+    uint8_t code[] = {WASM_I32V_2(kReturnValue)};
     EMIT_CODE_WITH_END(f, code);
     f->SetCompilationHint(WasmCompilationHintStrategy::kLazy,
                           WasmCompilationHintTier::kBaseline,
@@ -122,8 +122,8 @@ TEST(Run_WasmModule_CompilationHintsLazy) {
     HandleScope scope(isolate);
     testing::SetupIsolateForWasmModule(isolate);
     ErrorThrower thrower(isolate, "CompileAndRunWasmModule");
-    MaybeHandle<WasmModuleObject> module = testing::CompileForTesting(
-        isolate, &thrower, ModuleWireBytes(buffer.begin(), buffer.end()));
+    MaybeDirectHandle<WasmModuleObject> module =
+        testing::CompileForTesting(isolate, &thrower, base::VectorOf(buffer));
     CHECK(!module.is_null());
 
     // Lazy function was not invoked and therefore not compiled yet.
@@ -134,11 +134,12 @@ TEST(Run_WasmModule_CompilationHintsLazy) {
     CHECK(compilation_state->baseline_compilation_finished());
 
     // Instantiate and invoke function.
-    MaybeHandle<WasmInstanceObject> instance = GetWasmEngine()->SyncInstantiate(
-        isolate, &thrower, module.ToHandleChecked(), {}, {});
+    MaybeDirectHandle<WasmInstanceObject> instance =
+        GetWasmEngine()->SyncInstantiate(isolate, &thrower,
+                                         module.ToHandleChecked(), {}, {});
     CHECK(!instance.is_null());
     int32_t result = testing::CallWasmFunctionForTesting(
-        isolate, instance.ToHandleChecked(), "main", 0, nullptr);
+        isolate, instance.ToHandleChecked(), "main", {});
     CHECK_EQ(kReturnValue, result);
 
     // Lazy function was invoked and therefore compiled.
@@ -155,7 +156,8 @@ TEST(Run_WasmModule_CompilationHintsLazy) {
 }
 
 TEST(Run_WasmModule_CompilationHintsNoTiering) {
-  if (!FLAG_wasm_tier_up || !FLAG_liftoff) return;
+  FlagScope<bool> no_lazy_compilation(&v8_flags.wasm_lazy_compilation, false);
+  if (!v8_flags.wasm_tier_up || !v8_flags.liftoff) return;
   {
     EXPERIMENTAL_FLAG_SCOPE(compilation_hints);
 
@@ -168,7 +170,7 @@ TEST(Run_WasmModule_CompilationHintsNoTiering) {
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_I32V_2(kReturnValue)};
+    uint8_t code[] = {WASM_I32V_2(kReturnValue)};
     EMIT_CODE_WITH_END(f, code);
     f->SetCompilationHint(WasmCompilationHintStrategy::kEager,
                           WasmCompilationHintTier::kBaseline,
@@ -181,8 +183,8 @@ TEST(Run_WasmModule_CompilationHintsNoTiering) {
     HandleScope scope(isolate);
     testing::SetupIsolateForWasmModule(isolate);
     ErrorThrower thrower(isolate, "CompileAndRunWasmModule");
-    MaybeHandle<WasmModuleObject> module = testing::CompileForTesting(
-        isolate, &thrower, ModuleWireBytes(buffer.begin(), buffer.end()));
+    MaybeDirectHandle<WasmModuleObject> module =
+        testing::CompileForTesting(isolate, &thrower, base::VectorOf(buffer));
     CHECK(!module.is_null());
 
     // Synchronous compilation finished and no tiering units were initialized.
@@ -195,14 +197,15 @@ TEST(Run_WasmModule_CompilationHintsNoTiering) {
     CHECK_EQ(expected_tier, actual_tier);
     auto* compilation_state = native_module->compilation_state();
     CHECK(compilation_state->baseline_compilation_finished());
-    CHECK(compilation_state->top_tier_compilation_finished());
   }
   Cleanup();
 }
 
 TEST(Run_WasmModule_CompilationHintsTierUp) {
-  FlagScope<bool> no_wasm_dynamic_tiering(&FLAG_wasm_dynamic_tiering, false);
-  if (!FLAG_wasm_tier_up || !FLAG_liftoff) return;
+  FlagScope<bool> no_wasm_dynamic_tiering(&v8_flags.wasm_dynamic_tiering,
+                                          false);
+  FlagScope<bool> no_lazy_compilation(&v8_flags.wasm_lazy_compilation, false);
+  if (!v8_flags.wasm_tier_up || !v8_flags.liftoff) return;
   {
     EXPERIMENTAL_FLAG_SCOPE(compilation_hints);
 
@@ -215,7 +218,7 @@ TEST(Run_WasmModule_CompilationHintsTierUp) {
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_I32V_2(kReturnValue)};
+    uint8_t code[] = {WASM_I32V_2(kReturnValue)};
     EMIT_CODE_WITH_END(f, code);
     f->SetCompilationHint(WasmCompilationHintStrategy::kEager,
                           WasmCompilationHintTier::kBaseline,
@@ -228,8 +231,8 @@ TEST(Run_WasmModule_CompilationHintsTierUp) {
     HandleScope scope(isolate);
     testing::SetupIsolateForWasmModule(isolate);
     ErrorThrower thrower(isolate, "CompileAndRunWasmModule");
-    MaybeHandle<WasmModuleObject> module = testing::CompileForTesting(
-        isolate, &thrower, ModuleWireBytes(buffer.begin(), buffer.end()));
+    MaybeDirectHandle<WasmModuleObject> module =
+        testing::CompileForTesting(isolate, &thrower, base::VectorOf(buffer));
     CHECK(!module.is_null());
 
     // Expect baseline or top tier code.
@@ -247,27 +250,24 @@ TEST(Run_WasmModule_CompilationHintsTierUp) {
       CHECK(compilation_state->baseline_compilation_finished());
     }
 
-    // Busy wait for top tier compilation to finish.
-    while (!compilation_state->top_tier_compilation_finished()) {
-    }
-
-    // Expect top tier code.
+    // Tier-up is happening in the background. Eventually we should have top
+    // tier code.
     ExecutionTier top_tier = ExecutionTier::kTurbofan;
-    {
+    ExecutionTier actual_tier = ExecutionTier::kNone;
+    while (actual_tier != top_tier) {
       CHECK(native_module->HasCode(kFuncIndex));
       WasmCodeRefScope code_ref_scope;
-      ExecutionTier actual_tier = native_module->GetCode(kFuncIndex)->tier();
-      CHECK_EQ(top_tier, actual_tier);
-      CHECK(compilation_state->baseline_compilation_finished());
-      CHECK(compilation_state->top_tier_compilation_finished());
+      actual_tier = native_module->GetCode(kFuncIndex)->tier();
     }
   }
   Cleanup();
 }
 
 TEST(Run_WasmModule_CompilationHintsLazyBaselineEagerTopTier) {
-  FlagScope<bool> no_wasm_dynamic_tiering(&FLAG_wasm_dynamic_tiering, false);
-  if (!FLAG_wasm_tier_up || !FLAG_liftoff) return;
+  FlagScope<bool> no_wasm_dynamic_tiering(&v8_flags.wasm_dynamic_tiering,
+                                          false);
+  FlagScope<bool> no_lazy_compilation(&v8_flags.wasm_lazy_compilation, false);
+  if (!v8_flags.wasm_tier_up || !v8_flags.liftoff) return;
   {
     EXPERIMENTAL_FLAG_SCOPE(compilation_hints);
 
@@ -280,7 +280,7 @@ TEST(Run_WasmModule_CompilationHintsLazyBaselineEagerTopTier) {
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_I32V_2(kReturnValue)};
+    uint8_t code[] = {WASM_I32V_2(kReturnValue)};
     EMIT_CODE_WITH_END(f, code);
     f->SetCompilationHint(
         WasmCompilationHintStrategy::kLazyBaselineEagerTopTier,
@@ -294,30 +294,26 @@ TEST(Run_WasmModule_CompilationHintsLazyBaselineEagerTopTier) {
     HandleScope scope(isolate);
     testing::SetupIsolateForWasmModule(isolate);
     ErrorThrower thrower(isolate, "CompileAndRunWasmModule");
-    MaybeHandle<WasmModuleObject> module = testing::CompileForTesting(
-        isolate, &thrower, ModuleWireBytes(buffer.begin(), buffer.end()));
+    MaybeDirectHandle<WasmModuleObject> module =
+        testing::CompileForTesting(isolate, &thrower, base::VectorOf(buffer));
     CHECK(!module.is_null());
 
     NativeModule* native_module = module.ToHandleChecked()->native_module();
     auto* compilation_state = native_module->compilation_state();
 
-    // Busy wait for top tier compilation to finish.
-    while (!compilation_state->top_tier_compilation_finished()) {
-    }
-
-    // Expect top tier code.
+    // We have no code initially (because of lazy baseline), but eventually we
+    // should have TurboFan ready (because of eager top tier).
     static_assert(ExecutionTier::kLiftoff < ExecutionTier::kTurbofan,
                   "Assume an order on execution tiers");
-    static const int kFuncIndex = 0;
-    ExecutionTier top_tier = ExecutionTier::kTurbofan;
-    {
-      CHECK(native_module->HasCode(kFuncIndex));
-      WasmCodeRefScope code_ref_scope;
-      ExecutionTier actual_tier = native_module->GetCode(kFuncIndex)->tier();
-      CHECK_EQ(top_tier, actual_tier);
-      CHECK(compilation_state->baseline_compilation_finished());
-      CHECK(compilation_state->top_tier_compilation_finished());
+    constexpr int kFuncIndex = 0;
+    WasmCodeRefScope code_ref_scope;
+    while (true) {
+      auto* function_code = native_module->GetCode(kFuncIndex);
+      if (!function_code) continue;
+      CHECK_EQ(ExecutionTier::kTurbofan, function_code->tier());
+      break;
     }
+    CHECK(compilation_state->baseline_compilation_finished());
   }
   Cleanup();
 }
@@ -333,14 +329,14 @@ TEST(Run_WasmModule_CallAdd) {
     WasmFunctionBuilder* f1 = builder->AddFunction(sigs.i_ii());
     uint16_t param1 = 0;
     uint16_t param2 = 1;
-    byte code1[] = {
+    uint8_t code1[] = {
         WASM_I32_ADD(WASM_LOCAL_GET(param1), WASM_LOCAL_GET(param2))};
     EMIT_CODE_WITH_END(f1, code1);
 
     WasmFunctionBuilder* f2 = builder->AddFunction(sigs.i_v());
 
     ExportAsMain(f2);
-    byte code2[] = {
+    uint8_t code2[] = {
         WASM_CALL_FUNCTION(f1->func_index(), WASM_I32V_2(77), WASM_I32V_1(22))};
     EMIT_CODE_WITH_END(f2, code2);
     TestModule(&zone, builder, 99);
@@ -350,19 +346,20 @@ TEST(Run_WasmModule_CallAdd) {
 
 TEST(Run_WasmModule_ReadLoadedDataSegment) {
   {
-    static const byte kDataSegmentDest0 = 12;
+    static const uint8_t kDataSegmentDest0 = 12;
     v8::internal::AccountingAllocator allocator;
     Zone zone(&allocator, ZONE_NAME);
     TestSignatures sigs;
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
 
     ExportAsMain(f);
-    byte code[] = {
+    uint8_t code[] = {
         WASM_LOAD_MEM(MachineType::Int32(), WASM_I32V_1(kDataSegmentDest0))};
     EMIT_CODE_WITH_END(f, code);
-    byte data[] = {0xAA, 0xBB, 0xCC, 0xDD};
+    uint8_t data[] = {0xAA, 0xBB, 0xCC, 0xDD};
     builder->AddDataSegment(data, sizeof(data), kDataSegmentDest0);
     TestModule(&zone, builder, 0xDDCCBBAA);
   }
@@ -377,11 +374,12 @@ TEST(Run_WasmModule_CheckMemoryIsZero) {
     TestSignatures sigs;
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
 
     uint16_t localIndex = f->AddLocal(kWasmI32);
     ExportAsMain(f);
-    byte code[] = {WASM_BLOCK_I(
+    uint8_t code[] = {WASM_BLOCK_I(
         WASM_WHILE(
             WASM_I32_LTS(WASM_LOCAL_GET(localIndex), WASM_I32V_3(kCheckSize)),
             WASM_IF_ELSE(
@@ -402,11 +400,12 @@ TEST(Run_WasmModule_CallMain_recursive) {
     TestSignatures sigs;
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
 
     uint16_t localIndex = f->AddLocal(kWasmI32);
     ExportAsMain(f);
-    byte code[] = {
+    uint8_t code[] = {
         WASM_LOCAL_SET(localIndex,
                        WASM_LOAD_MEM(MachineType::Int32(), WASM_ZERO)),
         WASM_IF_ELSE_I(WASM_I32_LTS(WASM_LOCAL_GET(localIndex), WASM_I32V_1(5)),
@@ -427,17 +426,17 @@ TEST(Run_WasmModule_Global) {
     TestSignatures sigs;
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
-    uint32_t global1 = builder->AddGlobal(kWasmI32);
-    uint32_t global2 = builder->AddGlobal(kWasmI32);
+    uint32_t global1 = builder->AddGlobal(kWasmI32, true, WasmInitExpr(0));
+    uint32_t global2 = builder->AddGlobal(kWasmI32, true, WasmInitExpr(0));
     WasmFunctionBuilder* f1 = builder->AddFunction(sigs.i_v());
-    byte code1[] = {
+    uint8_t code1[] = {
         WASM_I32_ADD(WASM_GLOBAL_GET(global1), WASM_GLOBAL_GET(global2))};
     EMIT_CODE_WITH_END(f1, code1);
     WasmFunctionBuilder* f2 = builder->AddFunction(sigs.i_v());
     ExportAsMain(f2);
-    byte code2[] = {WASM_GLOBAL_SET(global1, WASM_I32V_1(56)),
-                    WASM_GLOBAL_SET(global2, WASM_I32V_1(41)),
-                    WASM_RETURN(WASM_CALL_FUNCTION0(f1->func_index()))};
+    uint8_t code2[] = {WASM_GLOBAL_SET(global1, WASM_I32V_1(56)),
+                       WASM_GLOBAL_SET(global2, WASM_I32V_1(41)),
+                       WASM_RETURN(WASM_CALL_FUNCTION0(f1->func_index()))};
     EMIT_CODE_WITH_END(f2, code2);
     TestModule(&zone, builder, 97);
   }
@@ -453,9 +452,10 @@ TEST(MemorySize) {
     Zone zone(&allocator, ZONE_NAME);
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_MEMORY_SIZE};
+    uint8_t code[] = {WASM_MEMORY_SIZE};
     EMIT_CODE_WITH_END(f, code);
     TestModule(&zone, builder, kExpectedValue);
   }
@@ -471,10 +471,11 @@ TEST(Run_WasmModule_MemSize_GrowMem) {
     Zone zone(&allocator, ZONE_NAME);
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_MEMORY_GROW(WASM_I32V_1(10)), WASM_DROP,
-                   WASM_MEMORY_SIZE};
+    uint8_t code[] = {WASM_MEMORY_GROW(WASM_I32V_1(10)), WASM_DROP,
+                      WASM_MEMORY_SIZE};
     EMIT_CODE_WITH_END(f, code);
     TestModule(&zone, builder, kExpectedValue);
   }
@@ -490,9 +491,10 @@ TEST(MemoryGrowZero) {
     Zone zone(&allocator, ZONE_NAME);
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_MEMORY_GROW(WASM_I32V(0))};
+    uint8_t code[] = {WASM_MEMORY_GROW(WASM_I32V(0))};
     EMIT_CODE_WITH_END(f, code);
     TestModule(&zone, builder, kExpectedValue);
   }
@@ -533,7 +535,7 @@ class InterruptThread : public v8::base::Thread {
 TEST(TestInterruptLoop) {
   {
     // Do not dump the module of this test because it contains an infinite loop.
-    if (FLAG_dump_wasm_module) return;
+    if (v8_flags.dump_wasm_module) return;
 
     // This test tests that WebAssembly loops can be interrupted, i.e. that if
     // an
@@ -553,9 +555,10 @@ TEST(TestInterruptLoop) {
     Zone zone(&allocator, ZONE_NAME);
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {
+    uint8_t code[] = {
         WASM_LOOP(
             WASM_IF(WASM_NOT(WASM_LOAD_MEM(
                         MachineType::Int32(),
@@ -571,19 +574,20 @@ TEST(TestInterruptLoop) {
     HandleScope scope(isolate);
     testing::SetupIsolateForWasmModule(isolate);
     ErrorThrower thrower(isolate, "Test");
-    const Handle<WasmInstanceObject> instance =
-        CompileAndInstantiateForTesting(
-            isolate, &thrower, ModuleWireBytes(buffer.begin(), buffer.end()))
+    const DirectHandle<WasmInstanceObject> instance =
+        CompileAndInstantiateForTesting(isolate, &thrower,
+                                        base::VectorOf(buffer))
             .ToHandleChecked();
 
-    Handle<JSArrayBuffer> memory(instance->memory_object().array_buffer(),
-                                 isolate);
+    DirectHandle<JSArrayBuffer> memory(
+        instance->trusted_data(isolate)->memory_object(0)->array_buffer(),
+        isolate);
     std::atomic<int32_t>* memory_array =
         reinterpret_cast<std::atomic<int32_t>*>(memory->backing_store());
 
     InterruptThread thread(isolate, memory_array);
     CHECK(thread.Start());
-    testing::CallWasmFunctionForTesting(isolate, instance, "main", 0, nullptr);
+    testing::CallWasmFunctionForTesting(isolate, instance, "main", {});
     Address address = reinterpret_cast<Address>(
         &memory_array[InterruptThread::interrupt_location_]);
     CHECK_EQ(InterruptThread::interrupt_value_,
@@ -598,10 +602,11 @@ TEST(Run_WasmModule_MemoryGrowInIf) {
     v8::internal::AccountingAllocator allocator;
     Zone zone(&allocator, ZONE_NAME);
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_IF_ELSE_I(WASM_I32V(0), WASM_MEMORY_GROW(WASM_I32V(1)),
-                                  WASM_I32V(12))};
+    uint8_t code[] = {WASM_IF_ELSE_I(
+        WASM_I32V(0), WASM_MEMORY_GROW(WASM_I32V(1)), WASM_I32V(12))};
     EMIT_CODE_WITH_END(f, code);
     TestModule(&zone, builder, 12);
   }
@@ -621,9 +626,9 @@ TEST(Run_WasmModule_GrowMemOobOffset) {
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_v());
     ExportAsMain(f);
-    byte code[] = {WASM_MEMORY_GROW(WASM_I32V_1(1)),
-                   WASM_STORE_MEM(MachineType::Int32(), WASM_I32V(index),
-                                  WASM_I32V(value))};
+    uint8_t code[] = {WASM_MEMORY_GROW(WASM_I32V_1(1)),
+                      WASM_STORE_MEM(MachineType::Int32(), WASM_I32V(index),
+                                     WASM_I32V(value))};
     EMIT_CODE_WITH_END(f, code);
     TestModuleException(&zone, builder);
   }
@@ -641,12 +646,13 @@ TEST(Run_WasmModule_GrowMemOobFixedIndex) {
     Zone zone(isolate->allocator(), ZONE_NAME);
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_i());
     ExportAsMain(f);
-    byte code[] = {WASM_MEMORY_GROW(WASM_LOCAL_GET(0)), WASM_DROP,
-                   WASM_STORE_MEM(MachineType::Int32(), WASM_I32V(index),
-                                  WASM_I32V(value)),
-                   WASM_LOAD_MEM(MachineType::Int32(), WASM_I32V(index))};
+    uint8_t code[] = {WASM_MEMORY_GROW(WASM_LOCAL_GET(0)), WASM_DROP,
+                      WASM_STORE_MEM(MachineType::Int32(), WASM_I32V(index),
+                                     WASM_I32V(value)),
+                      WASM_LOAD_MEM(MachineType::Int32(), WASM_I32V(index))};
     EMIT_CODE_WITH_END(f, code);
 
     HandleScope scope(isolate);
@@ -655,24 +661,26 @@ TEST(Run_WasmModule_GrowMemOobFixedIndex) {
     testing::SetupIsolateForWasmModule(isolate);
 
     ErrorThrower thrower(isolate, "Test");
-    Handle<WasmInstanceObject> instance =
-        CompileAndInstantiateForTesting(
-            isolate, &thrower, ModuleWireBytes(buffer.begin(), buffer.end()))
+    DirectHandle<WasmInstanceObject> instance =
+        CompileAndInstantiateForTesting(isolate, &thrower,
+                                        base::VectorOf(buffer))
             .ToHandleChecked();
 
     // Initial memory size is 16 pages, should trap till index > MemSize on
     // consecutive GrowMem calls
     for (uint32_t i = 1; i < 5; i++) {
-      Handle<Object> params[1] = {Handle<Object>(Smi::FromInt(i), isolate)};
+      DirectHandle<Object> params[1] = {
+          direct_handle(Smi::FromInt(i), isolate)};
       v8::TryCatch try_catch(reinterpret_cast<v8::Isolate*>(isolate));
-      testing::CallWasmFunctionForTesting(isolate, instance, "main", 1, params);
+      testing::CallWasmFunctionForTesting(isolate, instance, "main",
+                                          base::ArrayVector(params));
       CHECK(try_catch.HasCaught());
-      isolate->clear_pending_exception();
+      isolate->clear_exception();
     }
 
-    Handle<Object> params[1] = {Handle<Object>(Smi::FromInt(1), isolate)};
-    int32_t result = testing::CallWasmFunctionForTesting(isolate, instance,
-                                                         "main", 1, params);
+    DirectHandle<Object> params[1] = {direct_handle(Smi::FromInt(1), isolate)};
+    int32_t result = testing::CallWasmFunctionForTesting(
+        isolate, instance, "main", base::ArrayVector(params));
     CHECK_EQ(0xACED, result);
   }
   Cleanup();
@@ -688,12 +696,13 @@ TEST(Run_WasmModule_GrowMemOobVariableIndex) {
     Zone zone(&allocator, ZONE_NAME);
 
     WasmModuleBuilder* builder = zone.New<WasmModuleBuilder>(&zone);
+    builder->AddMemory(16);
     WasmFunctionBuilder* f = builder->AddFunction(sigs.i_i());
     ExportAsMain(f);
-    byte code[] = {WASM_MEMORY_GROW(WASM_I32V_1(1)), WASM_DROP,
-                   WASM_STORE_MEM(MachineType::Int32(), WASM_LOCAL_GET(0),
-                                  WASM_I32V(value)),
-                   WASM_LOAD_MEM(MachineType::Int32(), WASM_LOCAL_GET(0))};
+    uint8_t code[] = {WASM_MEMORY_GROW(WASM_I32V_1(1)), WASM_DROP,
+                      WASM_STORE_MEM(MachineType::Int32(), WASM_LOCAL_GET(0),
+                                     WASM_I32V(value)),
+                      WASM_LOAD_MEM(MachineType::Int32(), WASM_LOCAL_GET(0))};
     EMIT_CODE_WITH_END(f, code);
 
     HandleScope scope(isolate);
@@ -702,36 +711,38 @@ TEST(Run_WasmModule_GrowMemOobVariableIndex) {
     testing::SetupIsolateForWasmModule(isolate);
 
     ErrorThrower thrower(isolate, "Test");
-    Handle<WasmInstanceObject> instance =
-        CompileAndInstantiateForTesting(
-            isolate, &thrower, ModuleWireBytes(buffer.begin(), buffer.end()))
+    DirectHandle<WasmInstanceObject> instance =
+        CompileAndInstantiateForTesting(isolate, &thrower,
+                                        base::VectorOf(buffer))
             .ToHandleChecked();
 
     // Initial memory size is 16 pages, should trap till index > MemSize on
     // consecutive GrowMem calls
     for (int i = 1; i < 5; i++) {
-      Handle<Object> params[1] = {
-          Handle<Object>(Smi::FromInt((16 + i) * kPageSize - 3), isolate)};
+      DirectHandle<Object> params[] = {
+          direct_handle(Smi::FromInt((16 + i) * kPageSize - 3), isolate)};
       v8::TryCatch try_catch(reinterpret_cast<v8::Isolate*>(isolate));
-      testing::CallWasmFunctionForTesting(isolate, instance, "main", 1, params);
+      testing::CallWasmFunctionForTesting(isolate, instance, "main",
+                                          base::ArrayVector(params));
       CHECK(try_catch.HasCaught());
-      isolate->clear_pending_exception();
+      isolate->clear_exception();
     }
 
     for (int i = 1; i < 5; i++) {
-      Handle<Object> params[1] = {
-          Handle<Object>(Smi::FromInt((20 + i) * kPageSize - 4), isolate)};
-      int32_t result = testing::CallWasmFunctionForTesting(isolate, instance,
-                                                           "main", 1, params);
+      DirectHandle<Object> params[] = {
+          direct_handle(Smi::FromInt((20 + i) * kPageSize - 4), isolate)};
+      int32_t result = testing::CallWasmFunctionForTesting(
+          isolate, instance, "main", base::ArrayVector(params));
       CHECK_EQ(0xACED, result);
     }
 
     v8::TryCatch try_catch(reinterpret_cast<v8::Isolate*>(isolate));
-    Handle<Object> params[1] = {
-        Handle<Object>(Smi::FromInt(25 * kPageSize), isolate)};
-    testing::CallWasmFunctionForTesting(isolate, instance, "main", 1, params);
+    DirectHandle<Object> params[] = {
+        direct_handle(Smi::FromInt(25 * kPageSize), isolate)};
+    testing::CallWasmFunctionForTesting(isolate, instance, "main",
+                                        base::ArrayVector(params));
     CHECK(try_catch.HasCaught());
-    isolate->clear_pending_exception();
+    isolate->clear_exception();
   }
   Cleanup();
 }
@@ -748,7 +759,7 @@ TEST(Run_WasmModule_Global_init) {
     uint32_t global2 =
         builder->AddGlobal(kWasmI32, false, WasmInitExpr(222222));
     WasmFunctionBuilder* f1 = builder->AddFunction(sigs.i_v());
-    byte code[] = {
+    uint8_t code[] = {
         WASM_I32_ADD(WASM_GLOBAL_GET(global1), WASM_GLOBAL_GET(global2))};
     EMIT_CODE_WITH_END(f1, code);
     ExportAsMain(f1);
@@ -762,7 +773,6 @@ static void RunWasmModuleGlobalInitTest(ValueType type, CType expected) {
   {
     v8::internal::AccountingAllocator allocator;
     Zone zone(&allocator, ZONE_NAME);
-    TestSignatures sigs;
 
     ValueType types[] = {type};
     FunctionSig sig(1, 0, types);
@@ -780,7 +790,7 @@ static void RunWasmModuleGlobalInitTest(ValueType type, CType expected) {
       }
 
       WasmFunctionBuilder* f1 = builder->AddFunction(&sig);
-      byte code[] = {WASM_GLOBAL_GET(global)};
+      uint8_t code[] = {WASM_GLOBAL_GET(global)};
       EMIT_CODE_WITH_END(f1, code);
       ExportAsMain(f1);
       TestModule(&zone, builder, expected);
@@ -812,7 +822,7 @@ TEST(InitDataAtTheUpperLimit) {
 
     ErrorThrower thrower(isolate, "Run_WasmModule_InitDataAtTheUpperLimit");
 
-    const byte data[] = {
+    const uint8_t data[] = {
         WASM_MODULE_HEADER,   // --
         kMemorySectionCode,   // --
         U32V_1(4),            // section size
@@ -830,10 +840,9 @@ TEST(InitDataAtTheUpperLimit) {
         'c'         // data bytes
     };
 
-    CompileAndInstantiateForTesting(
-        isolate, &thrower, ModuleWireBytes(data, data + arraysize(data)));
+    CompileAndInstantiateForTesting(isolate, &thrower, base::VectorOf(data));
     if (thrower.error()) {
-      thrower.Reify()->Print();
+      Print(*thrower.Reify());
       FATAL("compile or instantiate error");
     }
   }
@@ -848,7 +857,7 @@ TEST(EmptyMemoryNonEmptyDataSegment) {
 
     ErrorThrower thrower(isolate, "Run_WasmModule_InitDataAtTheUpperLimit");
 
-    const byte data[] = {
+    const uint8_t data[] = {
         WASM_MODULE_HEADER,  // --
         kMemorySectionCode,  // --
         U32V_1(4),           // section size
@@ -866,8 +875,7 @@ TEST(EmptyMemoryNonEmptyDataSegment) {
         'c'         // data bytes
     };
 
-    CompileAndInstantiateForTesting(
-        isolate, &thrower, ModuleWireBytes(data, data + arraysize(data)));
+    CompileAndInstantiateForTesting(isolate, &thrower, base::VectorOf(data));
     // It should not be possible to instantiate this module.
     CHECK(thrower.error());
   }
@@ -882,7 +890,7 @@ TEST(EmptyMemoryEmptyDataSegment) {
 
     ErrorThrower thrower(isolate, "Run_WasmModule_InitDataAtTheUpperLimit");
 
-    const byte data[] = {
+    const uint8_t data[] = {
         WASM_MODULE_HEADER,  // --
         kMemorySectionCode,  // --
         U32V_1(4),           // section size
@@ -899,8 +907,7 @@ TEST(EmptyMemoryEmptyDataSegment) {
         U32V_1(0),  // source size
     };
 
-    CompileAndInstantiateForTesting(
-        isolate, &thrower, ModuleWireBytes(data, data + arraysize(data)));
+    CompileAndInstantiateForTesting(isolate, &thrower, base::VectorOf(data));
     // It should be possible to instantiate this module.
     CHECK(!thrower.error());
   }
