@@ -7,8 +7,8 @@
 #include "src/codegen/assembler-inl.h"
 #include "src/objects/call-site-info-inl.h"
 #include "test/cctest/cctest.h"
-#include "test/cctest/compiler/value-helper.h"
 #include "test/cctest/wasm/wasm-run-utils.h"
+#include "test/common/value-helper.h"
 #include "test/common/wasm/test-signatures.h"
 #include "test/common/wasm/wasm-macro-gen.h"
 
@@ -54,10 +54,11 @@ struct ExceptionInfo {
 };
 
 template <int N>
-void CheckExceptionInfos(v8::internal::Isolate* i_isolate, Handle<Object> exc,
+void CheckExceptionInfos(v8::internal::Isolate* i_isolate,
+                         DirectHandle<Object> exc,
                          const ExceptionInfo (&excInfos)[N]) {
   // Check that it's indeed an Error object.
-  CHECK(exc->IsJSError());
+  CHECK(IsJSError(*exc));
 
   v8::Isolate* v8_isolate = reinterpret_cast<v8::Isolate*>(i_isolate);
 
@@ -87,13 +88,16 @@ void CheckExceptionInfos(v8::internal::Isolate* i_isolate, Handle<Object> exc,
                        stack->GetFrame(v8_isolate, 0));
 }
 
-void CheckComputeLocation(v8::internal::Isolate* i_isolate, Handle<Object> exc,
+void CheckComputeLocation(v8::internal::Isolate* i_isolate,
+                          DirectHandle<Object> exc,
                           const ExceptionInfo& topLocation,
                           const v8::Local<v8::StackFrame> stackFrame) {
   MessageLocation loc;
   CHECK(i_isolate->ComputeLocationFromSimpleStackTrace(&loc, exc));
   printf("loc start: %d, end: %d\n", loc.start_pos(), loc.end_pos());
-  Handle<JSMessageObject> message = i_isolate->CreateMessage(exc, nullptr);
+  DirectHandle<JSMessageObject> message =
+      i_isolate->CreateMessage(exc, nullptr);
+  JSMessageObject::EnsureSourcePositionsAvailable(i_isolate, message);
   printf("msg start: %d, end: %d, line: %d, col: %d\n",
          message->GetStartPosition(), message->GetEndPosition(),
          message->GetLineNumber(), message->GetColumnNumber());
@@ -106,12 +110,12 @@ void CheckComputeLocation(v8::internal::Isolate* i_isolate, Handle<Object> exc,
   //               whether Script::PositionInfo.column should be the offset
   //               relative to the module or relative to the function.
   // CHECK_EQ(topLocation.column - 1, message->GetColumnNumber());
-  String scriptSource = message->GetSource();
-  CHECK(scriptSource.IsString());
+  Tagged<String> scriptSource = message->GetSource();
+  CHECK(IsString(scriptSource));
   if (stackFrame->IsWasm()) {
-    CHECK_EQ(scriptSource.length(), 0);
+    CHECK_EQ(scriptSource->length(), 0);
   } else {
-    CHECK_GT(scriptSource.length(), 0);
+    CHECK_GT(scriptSource->length(), 0);
   }
 }
 
@@ -125,35 +129,35 @@ WASM_COMPILED_EXEC_TEST(CollectDetailedWasmStack_ExplicitThrowFromJs) {
   HandleScope scope(CcTest::InitIsolateOnce());
   const char* source =
       "(function js() {\n function a() {\n throw new Error(); };\n a(); })";
-  Handle<JSFunction> js_function =
-      Handle<JSFunction>::cast(v8::Utils::OpenHandle(
+  DirectHandle<JSFunction> js_function =
+      Cast<JSFunction>(v8::Utils::OpenDirectHandle(
           *v8::Local<v8::Function>::Cast(CompileRun(source))));
   ManuallyImportedJSFunction import = {sigs.v_v(), js_function};
   uint32_t js_throwing_index = 0;
-  WasmRunner<void> r(execution_tier, &import);
+  WasmRunner<void> r(execution_tier, kWasmOrigin, &import);
 
   // Add a nop such that we don't always get position 1.
-  BUILD(r, WASM_NOP, WASM_CALL_FUNCTION0(js_throwing_index));
+  r.Build({WASM_NOP, WASM_CALL_FUNCTION0(js_throwing_index)});
   uint32_t wasm_index_1 = r.function()->func_index;
 
   WasmFunctionCompiler& f2 = r.NewFunction<void>("call_main");
-  BUILD(f2, WASM_CALL_FUNCTION0(wasm_index_1));
+  f2.Build({WASM_CALL_FUNCTION0(wasm_index_1)});
   uint32_t wasm_index_2 = f2.function_index();
 
-  Handle<JSFunction> js_wasm_wrapper = r.builder().WrapCode(wasm_index_2);
+  DirectHandle<JSFunction> js_wasm_wrapper = r.builder().WrapCode(wasm_index_2);
 
-  Handle<JSFunction> js_trampoline = Handle<JSFunction>::cast(
-      v8::Utils::OpenHandle(*v8::Local<v8::Function>::Cast(
+  DirectHandle<JSFunction> js_trampoline = Cast<JSFunction>(
+      v8::Utils::OpenDirectHandle(*v8::Local<v8::Function>::Cast(
           CompileRun("(function callFn(fn) { fn(); })"))));
 
   Isolate* isolate = js_wasm_wrapper->GetIsolate();
   isolate->SetCaptureStackTraceForUncaughtExceptions(true, 10,
                                                      v8::StackTrace::kOverview);
-  Handle<Object> global(isolate->context().global_object(), isolate);
-  MaybeHandle<Object> maybe_exc;
-  Handle<Object> args[] = {js_wasm_wrapper};
-  MaybeHandle<Object> returnObjMaybe =
-      Execution::TryCall(isolate, js_trampoline, global, 1, args,
+  DirectHandle<Object> global(isolate->context()->global_object(), isolate);
+  MaybeDirectHandle<Object> maybe_exc;
+  DirectHandle<Object> args[] = {js_wasm_wrapper};
+  MaybeDirectHandle<Object> returnObjMaybe =
+      Execution::TryCall(isolate, js_trampoline, global, base::VectorOf(args),
                          Execution::MessageHandling::kReport, &maybe_exc);
   CHECK(returnObjMaybe.is_null());
 
@@ -171,19 +175,19 @@ WASM_COMPILED_EXEC_TEST(CollectDetailedWasmStack_ExplicitThrowFromJs) {
 // Trigger a trap in wasm, stack should contain a source url.
 WASM_COMPILED_EXEC_TEST(CollectDetailedWasmStack_WasmUrl) {
   // Create a WasmRunner with stack checks and traps enabled.
-  WasmRunner<int> r(execution_tier, nullptr, "main", kRuntimeExceptionSupport);
+  WasmRunner<int> r(execution_tier, kWasmOrigin, nullptr, "main");
 
-  std::vector<byte> trap_code(1, kExprUnreachable);
+  std::vector<uint8_t> trap_code(1, kExprUnreachable);
   r.Build(trap_code.data(), trap_code.data() + trap_code.size());
 
   WasmFunctionCompiler& f = r.NewFunction<int>("call_main");
-  BUILD(f, WASM_CALL_FUNCTION0(0));
+  f.Build({WASM_CALL_FUNCTION0(0)});
   uint32_t wasm_index = f.function_index();
 
-  Handle<JSFunction> js_wasm_wrapper = r.builder().WrapCode(wasm_index);
+  DirectHandle<JSFunction> js_wasm_wrapper = r.builder().WrapCode(wasm_index);
 
-  Handle<JSFunction> js_trampoline = Handle<JSFunction>::cast(
-      v8::Utils::OpenHandle(*v8::Local<v8::Function>::Cast(
+  DirectHandle<JSFunction> js_trampoline = Cast<JSFunction>(
+      v8::Utils::OpenDirectHandle(*v8::Local<v8::Function>::Cast(
           CompileRun("(function callFn(fn) { fn(); })"))));
 
   Isolate* isolate = js_wasm_wrapper->GetIsolate();
@@ -192,33 +196,34 @@ WASM_COMPILED_EXEC_TEST(CollectDetailedWasmStack_WasmUrl) {
 
   // Set the wasm script source url.
   const char* url = "http://example.com/example.wasm";
-  const Handle<String> source_url =
+  const DirectHandle<String> source_url =
       isolate->factory()->InternalizeUtf8String(url);
-  r.builder().instance_object()->module_object().script().set_source_url(
+  r.builder().instance_object()->module_object()->script()->set_source_url(
       *source_url);
 
   // Run the js wrapper.
-  Handle<Object> global(isolate->context().global_object(), isolate);
-  MaybeHandle<Object> maybe_exc;
-  Handle<Object> args[] = {js_wasm_wrapper};
-  MaybeHandle<Object> maybe_return_obj =
-      Execution::TryCall(isolate, js_trampoline, global, 1, args,
+  DirectHandle<Object> global(isolate->context()->global_object(), isolate);
+  MaybeDirectHandle<Object> maybe_exc;
+  DirectHandle<Object> args[] = {js_wasm_wrapper};
+  MaybeDirectHandle<Object> maybe_return_obj =
+      Execution::TryCall(isolate, js_trampoline, global, base::VectorOf(args),
                          Execution::MessageHandling::kReport, &maybe_exc);
 
   CHECK(maybe_return_obj.is_null());
-  Handle<Object> exception = maybe_exc.ToHandleChecked();
+  DirectHandle<Object> exception = maybe_exc.ToHandleChecked();
 
   // Extract stack trace from the exception.
-  Handle<FixedArray> stack_trace_object =
-      isolate->GetSimpleStackTrace(Handle<JSReceiver>::cast(exception));
+  DirectHandle<FixedArray> stack_trace_object =
+      isolate->GetSimpleStackTrace(Cast<JSReceiver>(exception));
   CHECK_NE(0, stack_trace_object->length());
-  Handle<CallSiteInfo> stack_frame(
-      CallSiteInfo::cast(stack_trace_object->get(0)), isolate);
+  DirectHandle<CallSiteInfo> stack_frame(
+      Cast<CallSiteInfo>(stack_trace_object->get(0)), isolate);
 
-  MaybeHandle<String> maybe_stack_trace_str =
+  MaybeDirectHandle<String> maybe_stack_trace_str =
       SerializeCallSiteInfo(isolate, stack_frame);
   CHECK(!maybe_stack_trace_str.is_null());
-  Handle<String> stack_trace_str = maybe_stack_trace_str.ToHandleChecked();
+  DirectHandle<String> stack_trace_str =
+      maybe_stack_trace_str.ToHandleChecked();
 
   // Check if the source_url is part of the stack trace.
   CHECK_NE(std::string(stack_trace_str->ToCString().get()).find(url),
@@ -230,38 +235,37 @@ WASM_COMPILED_EXEC_TEST(CollectDetailedWasmStack_WasmError) {
   for (int pos_shift = 0; pos_shift < 3; ++pos_shift) {
     // Test a position with 1, 2 or 3 bytes needed to represent it.
     int unreachable_pos = 1 << (8 * pos_shift);
-    TestSignatures sigs;
     // Create a WasmRunner with stack checks and traps enabled.
-    WasmRunner<int> r(execution_tier, nullptr, "main",
-                      kRuntimeExceptionSupport);
+    WasmRunner<int> r(execution_tier, kWasmOrigin, nullptr, "main");
 
-    std::vector<byte> trap_code(unreachable_pos + 1, kExprNop);
+    std::vector<uint8_t> trap_code(unreachable_pos + 1, kExprNop);
     trap_code[unreachable_pos] = kExprUnreachable;
     r.Build(trap_code.data(), trap_code.data() + trap_code.size());
 
     uint32_t wasm_index_1 = r.function()->func_index;
 
     WasmFunctionCompiler& f2 = r.NewFunction<int>("call_main");
-    BUILD(f2, WASM_CALL_FUNCTION0(0));
+    f2.Build({WASM_CALL_FUNCTION0(0)});
     uint32_t wasm_index_2 = f2.function_index();
 
-    Handle<JSFunction> js_wasm_wrapper = r.builder().WrapCode(wasm_index_2);
+    DirectHandle<JSFunction> js_wasm_wrapper =
+        r.builder().WrapCode(wasm_index_2);
 
-    Handle<JSFunction> js_trampoline = Handle<JSFunction>::cast(
-        v8::Utils::OpenHandle(*v8::Local<v8::Function>::Cast(
+    DirectHandle<JSFunction> js_trampoline = Cast<JSFunction>(
+        v8::Utils::OpenDirectHandle(*v8::Local<v8::Function>::Cast(
             CompileRun("(function callFn(fn) { fn(); })"))));
 
     Isolate* isolate = js_wasm_wrapper->GetIsolate();
     isolate->SetCaptureStackTraceForUncaughtExceptions(
         true, 10, v8::StackTrace::kOverview);
-    Handle<Object> global(isolate->context().global_object(), isolate);
-    MaybeHandle<Object> maybe_exc;
-    Handle<Object> args[] = {js_wasm_wrapper};
-    MaybeHandle<Object> maybe_return_obj =
-        Execution::TryCall(isolate, js_trampoline, global, 1, args,
+    DirectHandle<Object> global(isolate->context()->global_object(), isolate);
+    MaybeDirectHandle<Object> maybe_exc;
+    DirectHandle<Object> args[] = {js_wasm_wrapper};
+    MaybeDirectHandle<Object> maybe_return_obj =
+        Execution::TryCall(isolate, js_trampoline, global, base::VectorOf(args),
                            Execution::MessageHandling::kReport, &maybe_exc);
     CHECK(maybe_return_obj.is_null());
-    Handle<Object> exception = maybe_exc.ToHandleChecked();
+    DirectHandle<Object> exception = maybe_exc.ToHandleChecked();
 
     static constexpr int kMainLocalsLength = 1;
     const int main_offset =
