@@ -4,12 +4,18 @@
 
 #include "test/inspector/isolate-data.h"
 
+#include <optional>
+
+#include "include/libplatform/libplatform.h"
 #include "include/v8-context.h"
 #include "include/v8-exception.h"
 #include "include/v8-microtask-queue.h"
 #include "include/v8-template.h"
-#include "src/base/vector.h"
+#include "src/execution/isolate.h"
+#include "src/heap/heap.h"
+#include "src/init/v8.h"
 #include "src/inspector/test-interface.h"
+#include "test/inspector/frontend-channel.h"
 #include "test/inspector/task-runner.h"
 #include "test/inspector/utils.h"
 
@@ -53,8 +59,8 @@ InspectorIsolateData::InspectorIsolateData(
       v8::ArrayBuffer::Allocator::NewDefaultAllocator());
   params.array_buffer_allocator = array_buffer_allocator_.get();
   params.snapshot_blob = startup_data;
-  params.only_terminate_in_safe_scope = true;
   isolate_.reset(v8::Isolate::New(params));
+  v8::Isolate::Scope isolate_scope(isolate_.get());
   isolate_->SetMicrotasksPolicy(v8::MicrotasksPolicy::kScoped);
   if (with_inspector) {
     isolate_->AddMessageListener(&InspectorIsolateData::MessageHandler);
@@ -76,6 +82,34 @@ InspectorIsolateData* InspectorIsolateData::FromContext(
       context->GetAlignedPointerFromEmbedderData(kIsolateDataIndex));
 }
 
+InspectorIsolateData::~InspectorIsolateData() {
+  // Enter the isolate before destructing this InspectorIsolateData, so that
+  // destructors that run before the Isolate's destructor still see it as
+  // entered. Use a v8::Locker, in case the thread destroying the isolate is
+  // not the last one that entered it.
+  locker_.emplace(isolate());
+  isolate()->Enter();
+
+  // Sessions need to be deleted before channels can be cleaned up, and channels
+  // must be deleted before the isolate gets cleaned up. This means we first
+  // clean up all the sessions and immedatly after all the channels used by
+  // those sessions.
+  for (const auto& pair : sessions_) {
+    session_ids_for_cleanup_.insert(pair.first);
+  }
+
+  context_group_by_session_.clear();
+  sessions_.clear();
+
+  for (int session_id : session_ids_for_cleanup_) {
+    ChannelHolder::RemoveChannel(session_id);
+  }
+
+  // We don't care about completing pending per-isolate tasks, just delete
+  // them in case they still reference this Isolate.
+  v8::platform::NotifyIsolateShutdown(V8::GetCurrentPlatform(), isolate());
+}
+
 int InspectorIsolateData::CreateContextGroup() {
   int context_group_id = ++last_context_group_id_;
   if (!CreateContext(context_group_id, v8_inspector::StringView())) {
@@ -143,7 +177,7 @@ void InspectorIsolateData::RegisterModule(v8::Local<v8::Context> context,
 // static
 v8::MaybeLocal<v8::Module> InspectorIsolateData::ModuleResolveCallback(
     v8::Local<v8::Context> context, v8::Local<v8::String> specifier,
-    v8::Local<v8::FixedArray> import_assertions,
+    v8::Local<v8::FixedArray> import_attributes,
     v8::Local<v8::Module> referrer) {
   // TODO(v8:11189) Consider JSON modules support in the InspectorClient
   InspectorIsolateData* data = InspectorIsolateData::FromContext(context);
@@ -158,23 +192,65 @@ v8::MaybeLocal<v8::Module> InspectorIsolateData::ModuleResolveCallback(
   return maybe_module;
 }
 
-int InspectorIsolateData::ConnectSession(
+std::optional<int> InspectorIsolateData::ConnectSession(
     int context_group_id, const v8_inspector::StringView& state,
-    v8_inspector::V8Inspector::Channel* channel) {
+    std::unique_ptr<FrontendChannelImpl> channel, bool is_fully_trusted) {
+  if (contexts_.find(context_group_id) == contexts_.end()) return std::nullopt;
+
   v8::SealHandleScope seal_handle_scope(isolate());
   int session_id = ++last_session_id_;
-  sessions_[session_id] = inspector_->connect(context_group_id, channel, state);
+  // It's important that we register the channel before the `connect` as the
+  // inspector will already send notifications.
+  auto* c = channel.get();
+  ChannelHolder::AddChannel(session_id, std::move(channel));
+  sessions_[session_id] = inspector_->connect(
+      context_group_id, c, state,
+      is_fully_trusted ? v8_inspector::V8Inspector::kFullyTrusted
+                       : v8_inspector::V8Inspector::kUntrusted,
+      waiting_for_debugger_
+          ? v8_inspector::V8Inspector::kWaitingForDebugger
+          : v8_inspector::V8Inspector::kNotWaitingForDebugger);
   context_group_by_session_[sessions_[session_id].get()] = context_group_id;
   return session_id;
 }
 
-std::vector<uint8_t> InspectorIsolateData::DisconnectSession(int session_id) {
+namespace {
+
+class RemoveChannelTask : public TaskRunner::Task {
+ public:
+  explicit RemoveChannelTask(int session_id) : session_id_(session_id) {}
+  ~RemoveChannelTask() override = default;
+  bool is_priority_task() final { return false; }
+
+ private:
+  void Run(InspectorIsolateData* data) override {
+    ChannelHolder::RemoveChannel(session_id_);
+  }
+  int session_id_;
+};
+
+}  // namespace
+
+std::vector<uint8_t> InspectorIsolateData::DisconnectSession(
+    int session_id, TaskRunner* context_task_runner) {
   v8::SealHandleScope seal_handle_scope(isolate());
   auto it = sessions_.find(session_id);
   CHECK(it != sessions_.end());
   context_group_by_session_.erase(it->second.get());
   std::vector<uint8_t> result = it->second->state();
   sessions_.erase(it);
+
+  // The InspectorSession destructor does cleanup work like disabling agents.
+  // This could send some more notifications. We'll delay removing the channel
+  // so notification tasks have time to get sent.
+  // Note: This only works for tasks scheduled immediately by the desctructor.
+  //       Any task scheduled in turn by one of the "cleanup tasks" will run
+  //       AFTER the channel was removed.
+  context_task_runner->Append(std::make_unique<RemoveChannelTask>(session_id));
+
+  // In case we shutdown the test runner before the above task can run, we
+  // let the desctructor clean up the channel.
+  session_ids_for_cleanup_.insert(session_id);
   return result;
 }
 
@@ -195,6 +271,12 @@ void InspectorIsolateData::BreakProgram(
   }
 }
 
+void InspectorIsolateData::Stop(int session_id) {
+  v8::SealHandleScope seal_handle_scope(isolate());
+  auto it = sessions_.find(session_id);
+  if (it != sessions_.end()) it->second->stop();
+}
+
 void InspectorIsolateData::SchedulePauseOnNextStatement(
     int context_group_id, const v8_inspector::StringView& reason,
     const v8_inspector::StringView& details) {
@@ -333,6 +415,10 @@ void InspectorIsolateData::PromiseRejectHandler(v8::PromiseRejectMessage data) {
         v8_inspector::StringView(reinterpret_cast<const uint8_t*>(reason_str),
                                  strlen(reason_str)));
     return;
+  } else if (data.GetEvent() == v8::kPromiseRejectAfterResolved ||
+             data.GetEvent() == v8::kPromiseResolveAfterResolved) {
+    // Ignore reject/resolve after resolved, like the blink handler.
+    return;
   }
 
   v8::Local<v8::Value> exception = data.GetValue();
@@ -387,7 +473,7 @@ bool InspectorIsolateData::isInspectableHeapObject(
     v8::Local<v8::Object> object) {
   v8::Local<v8::Context> context = isolate()->GetCurrentContext();
   v8::MicrotasksScope microtasks_scope(
-      isolate(), v8::MicrotasksScope::kDoNotRunMicrotasks);
+      context, v8::MicrotasksScope::kDoNotRunMicrotasks);
   return !object->HasPrivate(context, not_inspectable_private_.Get(isolate()))
               .FromMaybe(false);
 }
@@ -404,7 +490,7 @@ void InspectorIsolateData::SetCurrentTimeMS(double time) {
 
 double InspectorIsolateData::currentTimeMS() {
   if (current_time_set_) return current_time_;
-  return V8::GetCurrentPlatform()->CurrentClockTimeMillis();
+  return V8::GetCurrentPlatform()->CurrentClockTimeMillisecondsHighResolution();
 }
 
 void InspectorIsolateData::SetMemoryInfo(v8::Local<v8::Value> memory_info) {
@@ -433,9 +519,21 @@ v8::MaybeLocal<v8::Value> InspectorIsolateData::memoryInfo(
 
 void InspectorIsolateData::runMessageLoopOnPause(int) {
   v8::SealHandleScope seal_handle_scope(isolate());
+  // Pumping the message loop below may trigger the execution of a stackless
+  // GC. We need to override the embedder stack state, to force scanning the
+  // stack, if this happens.
+  i::Heap* heap =
+      reinterpret_cast<i::Isolate*>(task_runner_->isolate())->heap();
+  i::EmbedderStackStateScope scope(
+      heap, i::EmbedderStackStateOrigin::kExplicitInvocation,
+      StackState::kMayContainHeapPointers);
   task_runner_->RunMessageLoop(true);
 }
 
+void InspectorIsolateData::runIfWaitingForDebugger(int) {
+  quitMessageLoopOnPause();
+}
+
 void InspectorIsolateData::quitMessageLoopOnPause() {
   v8::SealHandleScope seal_handle_scope(isolate());
   task_runner_->QuitMessageLoop();
@@ -447,8 +545,8 @@ void InspectorIsolateData::installAdditionalCommandLineAPI(
   CHECK(context->GetIsolate() == isolate());
   v8::HandleScope handle_scope(isolate());
   v8::Context::Scope context_scope(context);
-  v8::ScriptOrigin origin(isolate(), v8::String::NewFromUtf8Literal(
-                                         isolate(), "internal-console-api"));
+  v8::ScriptOrigin origin(
+      v8::String::NewFromUtf8Literal(isolate(), "internal-console-api"));
   v8::ScriptCompiler::Source scriptSource(
       additional_console_api_.Get(isolate()), origin);
   v8::MaybeLocal<v8::Script> script =
@@ -505,6 +603,13 @@ bool InspectorIsolateData::AssociateExceptionData(
       this->isolate()->GetCurrentContext(), exception, key, value);
 }
 
+void InspectorIsolateData::WaitForDebugger(int context_group_id) {
+  DCHECK(!waiting_for_debugger_);
+  waiting_for_debugger_ = true;
+  runMessageLoopOnPause(context_group_id);
+  waiting_for_debugger_ = false;
+}
+
 namespace {
 class StringBufferImpl : public v8_inspector::StringBuffer {
  public:
@@ -537,5 +642,27 @@ int64_t InspectorIsolateData::generateUniqueId() {
   return ++last_unique_id;
 }
 
+// static
+void ChannelHolder::AddChannel(int session_id,
+                               std::unique_ptr<FrontendChannelImpl> channel) {
+  CHECK_NE(channel.get(), nullptr);
+  channel->set_session_id(session_id);
+  channels_[session_id] = std::move(channel);
+}
+
+// static
+FrontendChannelImpl* ChannelHolder::GetChannel(int session_id) {
+  auto it = channels_.find(session_id);
+  return it != channels_.end() ? it->second.get() : nullptr;
+}
+
+// static
+void ChannelHolder::RemoveChannel(int session_id) {
+  channels_.erase(session_id);
+}
+
+// static
+std::map<int, std::unique_ptr<FrontendChannelImpl>> ChannelHolder::channels_;
+
 }  // namespace internal
 }  // namespace v8
