@@ -7,8 +7,8 @@
 #include "src/codegen/code-factory.h"
 #include "src/codegen/tick-counter.h"
 #include "src/compiler/compiler-source-position-table.h"
-#include "src/compiler/graph.h"
 #include "src/compiler/schedule.h"
+#include "src/compiler/turbofan-graph.h"
 #include "src/flags/flags.h"
 #include "src/objects/objects-inl.h"
 #include "test/unittests/compiler/compiler-test-utils.h"
@@ -17,9 +17,11 @@ namespace v8 {
 namespace internal {
 namespace compiler {
 
+// TODO(391750831): This needs to be ported to Turboshaft.
+#if 0
 InstructionSelectorTest::InstructionSelectorTest()
     : TestWithNativeContextAndZone(kCompressGraphZone),
-      rng_(FLAG_random_seed) {}
+      rng_(v8_flags.random_seed) {}
 
 InstructionSelectorTest::~InstructionSelectorTest() = default;
 
@@ -28,7 +30,7 @@ InstructionSelectorTest::Stream InstructionSelectorTest::StreamBuilder::Build(
     InstructionSelectorTest::StreamBuilderMode mode,
     InstructionSelector::SourcePositionMode source_position_mode) {
   Schedule* schedule = ExportForTest();
-  if (FLAG_trace_turbo) {
+  if (v8_flags.trace_turbo) {
     StdoutStream{} << "=== Schedule before instruction selection ==="
                    << std::endl
                    << *schedule;
@@ -44,7 +46,7 @@ InstructionSelectorTest::Stream InstructionSelectorTest::StreamBuilder::Build(
   TickCounter tick_counter;
   size_t max_unoptimized_frame_height = 0;
   size_t max_pushed_argument_count = 0;
-  InstructionSelector selector(
+  InstructionSelector selector = InstructionSelector::ForTurbofan(
       test_->zone(), node_count, &linkage, &sequence, schedule,
       &source_position_table, nullptr,
       InstructionSelector::kEnableSwitchJumpTable, &tick_counter, nullptr,
@@ -52,7 +54,7 @@ InstructionSelectorTest::Stream InstructionSelectorTest::StreamBuilder::Build(
       source_position_mode, features, InstructionSelector::kDisableScheduling,
       InstructionSelector::kEnableRootsRelativeAddressing);
   selector.SelectInstructions();
-  if (FLAG_trace_turbo) {
+  if (v8_flags.trace_turbo) {
     StdoutStream{} << "=== Code sequence after instruction selection ==="
                    << std::endl
                    << sequence;
@@ -154,10 +156,11 @@ bool InstructionSelectorTest::Stream::IsUsedAtStart(
 
 const FrameStateFunctionInfo*
 InstructionSelectorTest::StreamBuilder::GetFrameStateFunctionInfo(
-    int parameter_count, int local_count) {
+    uint16_t parameter_count, int local_count) {
+  const uint16_t max_arguments = 0;
   return common()->CreateFrameStateFunctionInfo(
-      FrameStateType::kUnoptimizedFunction, parameter_count, local_count,
-      Handle<SharedFunctionInfo>());
+      FrameStateType::kUnoptimizedFunction, parameter_count, max_arguments,
+      local_count, {}, {});
 }
 
 // -----------------------------------------------------------------------------
@@ -371,8 +374,16 @@ TARGET_TEST_F(InstructionSelectorTest, CallJSFunctionWithDeopt) {
       m.graph()->start());
 
   // Build the call.
-  Node* nodes[] = {function_node,      receiver, m.UndefinedConstant(),
-                   m.Int32Constant(1), context,  state_node};
+  Node* argc = m.Int32Constant(1);
+#ifdef V8_JS_LINKAGE_INCLUDES_DISPATCH_HANDLE
+  Node* dispatch_handle = m.Int32Constant(-1);
+  Node* nodes[] = {function_node, receiver,        m.UndefinedConstant(),
+                   argc,          dispatch_handle, context,
+                   state_node};
+#else
+  Node* nodes[] = {function_node, receiver, m.UndefinedConstant(),
+                   argc,          context,  state_node};
+#endif
   Node* call = m.CallNWithFrameState(call_descriptor, arraysize(nodes), nodes);
   m.Return(call);
 
@@ -438,7 +449,7 @@ TARGET_TEST_F(InstructionSelectorTest, CallStubWithDeopt) {
 
   Stream s = m.Build(kAllExceptNopInstructions);
 
-  // Skip until kArchCallJSFunction.
+  // Skip until kArchCallCodeObject.
   size_t index = 0;
   for (; index < s.size() && s[index]->arch_opcode() != kArchCallCodeObject;
        index++) {
@@ -453,7 +464,8 @@ TARGET_TEST_F(InstructionSelectorTest, CallStubWithDeopt) {
       1 +  // Code object.
       6 +  // Frame state deopt id + one input for each value in frame state.
       1 +  // Function.
-      1;   // Context.
+      1 +  // Context.
+      1;   // Entrypoint tag.
   ASSERT_EQ(num_operands, call_instr->InputCount());
 
   // Code object.
@@ -471,12 +483,14 @@ TARGET_TEST_F(InstructionSelectorTest, CallStubWithDeopt) {
   EXPECT_EQ(0, s.ToInt32(call_instr->InputAt(4)));  // This should be a context.
                                                     // We inserted 0 here.
   EXPECT_EQ(0.5, s.ToFloat64(call_instr->InputAt(5)));
-  EXPECT_TRUE(s.ToHeapObject(call_instr->InputAt(6))->IsUndefined(isolate()));
+  EXPECT_TRUE(IsUndefined(*s.ToHeapObject(call_instr->InputAt(6)), isolate()));
 
   // Function.
   EXPECT_EQ(s.ToVreg(function_node), s.ToVreg(call_instr->InputAt(7)));
   // Context.
   EXPECT_EQ(s.ToVreg(context), s.ToVreg(call_instr->InputAt(8)));
+  // Entrypoint tag.
+  EXPECT_TRUE(call_instr->InputAt(9)->IsImmediate());
 
   EXPECT_EQ(kArchRet, s[index++]->arch_opcode());
 
@@ -544,7 +558,7 @@ TARGET_TEST_F(InstructionSelectorTest, CallStubWithDeoptRecursiveFrameState) {
 
   Stream s = m.Build(kAllExceptNopInstructions);
 
-  // Skip until kArchCallJSFunction.
+  // Skip until kArchCallCodeObject.
   size_t index = 0;
   for (; index < s.size() && s[index]->arch_opcode() != kArchCallCodeObject;
        index++) {
@@ -561,7 +575,8 @@ TARGET_TEST_F(InstructionSelectorTest, CallStubWithDeoptRecursiveFrameState) {
       5 +  // One input for each value in frame state + context.
       5 +  // One input for each value in the parent frame state + context.
       1 +  // Function.
-      1;   // Context.
+      1 +  // Context.
+      1;   // Entrypoint tag.
   EXPECT_EQ(num_operands, call_instr->InputCount());
   // Code object.
   EXPECT_TRUE(call_instr->InputAt(0)->IsImmediate());
@@ -594,12 +609,14 @@ TARGET_TEST_F(InstructionSelectorTest, CallStubWithDeoptRecursiveFrameState) {
   EXPECT_EQ(s.ToVreg(function_node), s.ToVreg(call_instr->InputAt(12)));
   // Context.
   EXPECT_EQ(s.ToVreg(context2), s.ToVreg(call_instr->InputAt(13)));
+  // Entrypoint tag.
+  EXPECT_TRUE(call_instr->InputAt(14)->IsImmediate());
   // Continuation.
 
   EXPECT_EQ(kArchRet, s[index++]->arch_opcode());
   EXPECT_EQ(index, s.size());
 }
-
+#endif
 }  // namespace compiler
 }  // namespace internal
 }  // namespace v8
