@@ -11,6 +11,14 @@
 #include "src/base/page-allocator.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
+#ifdef V8_ENABLE_FUZZTEST
+#include "test/unittests/fuzztest-init-adapter.h"
+#endif  // V8_ENABLE_FUZZTEST
+
+#ifdef V8_USE_PERFETTO
+#include "src/tracing/trace-event.h"
+#endif  // V8_USE_PERFETTO
+
 namespace {
 
 class CppGCEnvironment final : public ::testing::Environment {
@@ -20,6 +28,13 @@ class CppGCEnvironment final : public ::testing::Environment {
     // has to survive as long as the process, so it's ok to leak the allocator
     // here.
     cppgc::InitializeProcess(new v8::base::PageAllocator());
+
+#ifdef V8_USE_PERFETTO
+    // Set up the in-process perfetto backend.
+    perfetto::TracingInitArgs init_args;
+    init_args.backends = perfetto::BackendType::kInProcessBackend;
+    perfetto::Tracing::Initialize(init_args);
+#endif  // V8_USE_PERFETTO
   }
 
   void TearDown() override { cppgc::ShutdownProcess(); }
@@ -31,15 +46,21 @@ class CppGCEnvironment final : public ::testing::Environment {
 int main(int argc, char** argv) {
   // Don't catch SEH exceptions and continue as the following tests might hang
   // in an broken environment on windows.
-  testing::GTEST_FLAG(catch_exceptions) = false;
+  GTEST_FLAG_SET(catch_exceptions, false);
 
   // Most V8 unit-tests are multi-threaded, so enable thread-safe death-tests.
-  testing::FLAGS_gtest_death_test_style = "threadsafe";
+  GTEST_FLAG_SET(death_test_style, "threadsafe");
 
   testing::InitGoogleMock(&argc, argv);
   testing::AddGlobalTestEnvironment(new CppGCEnvironment);
   v8::V8::SetFlagsFromCommandLine(&argc, argv, true);
   v8::V8::InitializeExternalStartupData(argv[0]);
   v8::V8::InitializeICUDefaultLocation(argv[0]);
+
+#ifdef V8_ENABLE_FUZZTEST
+  absl::ParseCommandLine(argc, argv);
+  fuzztest::InitFuzzTest(&argc, &argv);
+#endif  // V8_ENABLE_FUZZTEST
+
   return RUN_ALL_TESTS();
 }
